#coding=utf-8

import smtplib
from email.header import Header
from email.mime.text import MIMEText
from envelopes import Envelope
from email.header import Header


def send(
    from_addr,
    to_addr,
    host='localhost',
    port=25,
    cc_addr='',
    title='',
    content='',
    html_content='',
):
    if html_content.strip() == '':
        msg = MIMEText(content, 'plain', 'utf-8')
        msg['From'] = from_addr
        msg['To'] = to_addr
        msg['Cc'] = cc_addr
        msg['Subject'] = Header(title, 'utf-8').encode()
        s = smtplib.SMTP(host=host, port=port)
        s.sendmail(from_addr=from_addr, to_addrs=to_addr, msg=msg.as_string())
        return True
    else:
        envelope = Envelope(from_addr=tuple(from_addr.split(',')),
                            to_addr=tuple(to_addr.split(',')),
                            cc_addr=tuple(cc_addr.split(',')),
                            subject=title,
                            html_body=html_content)
        envelope.send(host=host)
        return True


def send_by_pass(from_addr,
                 password,
                 smtp_server,
                 smtp_port,
                 to_addr_list,
                 title,
                 content,
                 from_name=''):
    msg = MIMEText(content, 'plain', 'utf-8')
    msg['Subject'] = Header(title).encode()
    if from_name == '': from_name = from_addr
    msg['From'] = from_name
    server = smtplib.SMTP(smtp_server, smtp_port)
    server.set_debuglevel(1)
    server.login(from_addr, password)
    server.sendmail(from_addr, to_addr_list, msg.as_string())
    server.quit()



#coding=utf-8

from socket import *
import struct
from agileutil.sanic import SanicApp
from multiprocessing import cpu_count
import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning
from agileutil.rpc.const import SERVER_TIMEOUT


class RpcTransport(object): 

    def close(self):
        pass


class TcpTransport(RpcTransport):

    def __init__(self, host, port, timeout):
        RpcTransport.__init__(self)
        self.host = host
        self.port = port
        self.timeout = timeout
        self.socket = None
        self.keepaliveTimeout = SERVER_TIMEOUT

    def bind(self):
        self.socket = socket(AF_INET, SOCK_STREAM)
        self.socket.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
        self.socket.bind( (self.host, self.port) )
        self.socket.listen(100)

    def setKeepaliveTimeout(self, keepaliveTimeout: int):
        self.keepaliveTimeout = keepaliveTimeout

    def connect(self):
        if self.socket == None:
            self.socket = socket(AF_INET, SOCK_STREAM)
            self.socket.settimeout(self.timeout)
            self.socket.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
            self.socket.connect( (self.host, self.port) )

    def accept(self):
        cli, addr = self.socket.accept()
        cli.settimeout(self.keepaliveTimeout)
        return cli, addr

    def send(self, msg: bytes):
        newbyte = struct.pack("i", len(msg))
        try:
            self.socket.sendall(newbyte)
            self.socket.sendall(msg)
        except BrokenPipeError:
            self.reconnect()
            self.socket.sendall(newbyte)
            self.socket.sendall(msg)

    def sendPeer(self, msg: bytes, conn):
        newbyte = struct.pack("i", len(msg))
        conn.sendall(newbyte)
        conn.sendall(msg)

    def getSendByte(self, msg: bytes, conn = None):
        newbyte = struct.pack("i", len(msg))
        ret = newbyte + msg
        return ret

    def recv(self):
        #每次读一个完整的字节，再接收前4个字节，再取body
        conn = self.socket
        toread = 4
        readn = 0
        lengthbyte = b''
        while 1:
            bufsize = toread - readn
            if bufsize <= 0:
                break
            bytearr = conn.recv(bufsize)
            if len(bytearr) == 0:
                raise Exception('peer closed')
            lengthbyte = lengthbyte + bytearr
            readn = readn + len(bytearr)
        toread = struct.unpack("i", lengthbyte)[0]
        readn = 0
        msg = b''
        while 1:
            bufsize = toread - readn
            if bufsize <= 0:
                break
            bytearr = conn.recv(bufsize)
            if len(bytearr) == 0:
                raise Exception('peer closed')
            msg = msg + bytearr
            readn = readn + len(bytearr)
        return msg

    def recvPeer(self, conn):
        #每次读一个完整的字节，再接收前4个字节，再取body
        toread = 4
        readn = 0
        lengthbyte = b''
        while 1:
            bufsize = toread - readn
            if bufsize <= 0:
                break
            bytearr = conn.recv(bufsize)
            if len(bytearr) == 0:
                raise Exception('peer closed')
            lengthbyte = lengthbyte + bytearr
            readn = readn + len(bytearr)
        toread = struct.unpack("i", lengthbyte)[0]
        readn = 0
        msg = b''
        while 1:
            bufsize = toread - readn
            if bufsize <= 0:
                break
            bytearr = conn.recv(bufsize)
            if len(bytearr) == 0:
                raise Exception('peer closed')
            msg = msg + bytearr
            readn = readn + len(bytearr)
        return msg

    def close(self):
        self.socket.close()
        self.socket = None

    def reconnect(self):
        self.close()
        self.connect()
        print('tcp client reconnect')


class UdpTransport(RpcTransport):
    
    def __init__(self, host, port, timeout = SERVER_TIMEOUT):
        RpcTransport.__init__(self)
        self.host = host
        self.port = port
        self.socket = None
        self.timeout = timeout
        self.socket = socket(AF_INET, SOCK_DGRAM)
        self.socket.settimeout(self.timeout)
        self.socket.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)

    def bind(self):
        self.socket.bind( (self.host, self.port) )

    def sendPeer(self, msg: bytes, addr):
        self.socket.sendto(msg, addr)

    def send(self, msg: bytes):
        addr = (self.host, self.port)
        self.socket.sendto(msg, addr)

    def recv(self, conn = None):
        if conn == None:
            conn = self.socket
        msg, addr = conn.recvfrom(100000)
        return msg, addr

    def close(self):
        self.socket.close()
        self.socket = None


class ClientUdpTransport(UdpTransport):

    def __init__(self, host, port, timeout = 10):
        UdpTransport.__init__(self, host, port)
        self.timeout = timeout
        self.socket.settimeout(self.timeout)


class HttpTransport(RpcTransport):

    def __init__(self, host, port, worker=cpu_count(), timeout = 10, poolConnection=20, poolMaxSize=20, maxRetries=2):
        RpcTransport.__init__(self)
        self.host = host
        self.port = port
        self.worker = worker
        self.timeout = timeout
        self.app = SanicApp(host=host, port=port, worker_num=worker, debug=False)
        self.poolConnection = poolConnection
        self.poolMaxSize = poolMaxSize
        self.maxRetries = maxRetries
        self.requestSession = requests.Session()
        self.requestSession.mount('http://', requests.adapters.HTTPAdapter(pool_connections=self.poolConnection, pool_maxsize=self.poolMaxSize, max_retries=self.maxRetries))
        self.url = self.makeUrl()
        self.headers = {
            'Content-type' : 'application/octet-stream'
        }
        requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

    def makeUrl(self):
        return "http://%s:%s/" % (self.host, self.port)

    def send(self, msg):
        r = self.requestSession.post(self.url, headers = self.headers, data=msg, timeout = self.timeout)
        resp = r.content
        return resp


#coding=utf-8
'''
base on bsddb, key and value must be string or buffer
suport set, get, delete, has_key, setex, keys, values, flushall
'''

import time
try:
    import bsddb3 as bsddb
except:
    import bsddb


class Cache:
    def __init__(self, db_file):
        self._db_file = db_file
        self._db = bsddb.btopen(self._db_file, 'c')

    def set(self, k, v):
        k, v = str(k), str(v)
        self._db[k] = v
        return self

    def get(self, k, default_v=None):
        k = str(k)
        time_set_k = self._gen_time_set_key(k)
        time_expire_k = self._gen_time_expire_key(k)
        v = None
        time_set_v = None
        time_expire_v = None
        try:
            v = self._db[k]
        except:
            pass
        try:
            time_set_v = self._db[time_set_k]
        except:
            pass
        try:
            time_expire_v = self._db[time_expire_k]
        except:
            pass
        if time_set_v and time_expire_v:
            '''
			add by setex()
			'''
            set_time = float(time_set_v)
            expire_time = float(time_expire_v)
            if time.time() - set_time >= expire_time:
                self._db.pop(k)
                self._db.pop(time_set_k)
                self._db.pop(time_expire_k)
                return default_v
        if v == None:
            return default_v
        return v

    def has_key(self, k):
        if self.get(k):
            return True
        else:
            return False

    def delete(self, k):
        k = str(k)
        self._db.delete(k)
        return self

    def setex(self, k, v, seconds):
        k, v, seconds = str(k), str(v), int(seconds)
        time_set_k = self._gen_time_set_key(k)
        time_expire_k = self._gen_time_expire_key(k)
        cur_time = str(time.time())
        if seconds == 0: return
        if seconds < 0:
            self.set(k, v)
            return
        self.set(k, v)
        self.set(time_set_k, cur_time)
        self.set(time_expire_k, seconds)
        return self

    def _gen_time_set_key(self, k):
        return 'agileutil_cache_time_set_k_%s' % k

    def _gen_time_expire_key(self, k):
        return 'agileutil_cache_time_expire_k_%s' % k

    def keys(self):
        return filter(
            lambda x: 'agileutil_cache_time_set_k' not in x and
            'agileutil_cache_time_expire_k_' not in x, self._db.keys())

    def values(self):
        return filter(lambda x: x is not None,
                      [self.get(k) for k in self.keys()])

    def flushall(self):
        self._db.clear()
        return self

    def delete(self, k):
        k = str(k)
        return self._db.pop(k)



#coding=utf-8

import threading


class ThreadLock(object):
    """
    mutex lock
    """

    __threadLock = threading.Lock()

    @staticmethod
    def lock():
        ThreadLock.__threadLock.acquire()

    @staticmethod
    def unlock():
        ThreadLock.__threadLock.release()



#coding=utf-8

from agileutil.rpc.protocal import TcpProtocal, UdpProtocal, HttpProtocal, RpcProtocal, ClientUdpProtocal
from agileutil.rpc.exception import FuncNotFoundException
from agileutil.rpc.discovery import DiscoveryConfig
from agileutil.rpc.discovery import ConsulRpcDiscovery
from agileutil.wrap import retryTimes
from agileutil.rpc.discovery import Instance
import socket

class Address(object):

    def __init__(self, host, port):
        self.host = host
        self.port = port


class RpcClient(object): 

    def __init__(self):
        self.discoveryConfig = None
        self.discovery = None
        self.instanceIndex = 0
        self.protocalMap = {}
        self.isSync = False
        self.syncInterval = 30
        self.protocal = RpcProtocal()
        self.servers = []
        self.serversCount = len(self.servers)
        self.serversProtocalMap = {}
        self.serverIndex = 0
        self.lastServer = ''
        self.timeout = 10

    def getInstance(self):
        instanceList = self.discovery.getInstanceList(self.discoveryConfig.serviceName)
        instanceLength  = len(instanceList)
        if instanceLength == 0:
            raise Exception('no instance found')
        index = self.instanceIndex % instanceLength
        self.instanceIndex = self.instanceIndex + 1
        return instanceList[index]

    def refreshProtocal(self):
        if self.discovery:
            instance = self.getInstance()
            self.protocal = self.getProtocalInstance(instance)
        if self.servers:
            self.serversCount = len(self.servers)
            index = self.serverIndex % self.serversCount
            self.serverIndex = self.serverIndex + 1
            server = self.servers[ index ]
            self.lastServer = server
            self.protocal = self.serversProtocalMap[ server ]

    def getProtocalInstance(self, instance):
        #overide
        pass

    def setDiscoveryConfig(self, config: DiscoveryConfig):
        self.discoveryConfig = config
        self.discovery = ConsulRpcDiscovery(self.discoveryConfig.consulHost, self.discoveryConfig.consulPort)

    def close(self):
        self.protocal.transport.close()

    def __del__(self):
        self.close()

    def beforeCall(self):
        self.refreshProtocal()

    def debugGetLastServer(self):
        return self.lastServer


class TcpRpcClient(RpcClient):

    def __init__(self, host = '', port = 0, servers=[], timeout=10, keepconnect = True):
        RpcClient.__init__(self)
        self.host = host
        self.port = port
        self.keepconnect = keepconnect
        self.protocal = TcpProtocal(host, port, timeout=timeout)
        self.servers = servers
        self.timeout = timeout
        if len(self.servers) == 0 and self.host != '' and self.port != 0:
            self.servers = [ host+':'+str(port) ]
        for server in self.servers:
            host, port = server.split(':')
            self.serversProtocalMap[ server ] = TcpProtocal(host, int(port), timeout = timeout)

    @retryTimes(retryTimes=3)
    def call(self, func, args = ()):
        try:
            self.beforeCall()
            if args == None: args = ()
            self.protocal.transport.connect()
            package = {
                'func' : func,
                'args' : args,
            }
            msg = self.protocal.serialize(package)
            self.protocal.transport.send(msg)
            respmsg = self.protocal.transport.recv()
            resp = self.protocal.unserialize(respmsg)
            if isinstance(resp, Exception):
                if not self.keepconnect: self.protocal.transport.close()
                raise resp
            if not self.keepconnect: self.protocal.transport.close()
            return resp
        except socket.timeout as ex:
            self.protocal.transport.close()
            raise ex

    def getProtocalInstance(self, instance):
        key = "%s:%s:%s" % (instance.service, instance.address, instance.port)
        if key in self.protocalMap:
            return self.protocalMap.get(key)
        self.protocalMap[key] =  TcpProtocal(instance.address, instance.port)
        return self.protocalMap[key]


class UdpRpcClient(RpcClient):

    def __init__(self, host = '', port = 0, servers=[], timeout = 10):
        RpcClient.__init__(self)
        self.host = host
        self.port = port
        self.protocal = ClientUdpProtocal(host, port, timeout=timeout)
        self.servers = servers
        self.timeout = timeout
        if len(self.servers) == 0 and self.host != '' and self.port != 0:
            self.servers = [ host+':'+str(port) ]
        for server in self.servers:
            host, port = server.split(':')
            self.serversProtocalMap[ server ] = ClientUdpProtocal(host, int(port), timeout=timeout)

    @retryTimes(retryTimes=3)
    def call(self, func, args = ()):
        try:
            self.beforeCall()
            package = {
                'func' : func,
                'args' : args,
            }
            msg = self.protocal.serialize(package)
            self.protocal.transport.send(msg)
            respmsg, _ = self.protocal.transport.recv()
            resp = self.protocal.unserialize(respmsg)
            if isinstance(resp, Exception):
                raise resp
            return resp
        except socket.timeout as ex:
            self.protocal.newTransport()
            raise ex

    def getProtocalInstance(self, instance):
        key = "%s:%s:%s" % (instance.service, instance.address, instance.port)
        if key in self.protocalMap:
            return self.protocalMap.get(key)
        self.protocalMap[key] =  UdpProtocal(instance.address, instance.port)
        return self.protocalMap[key]


class HttpRpcClient(RpcClient):

    def __init__(self, host = '', port = 0, servers = [], timeout = 10):
        RpcClient.__init__(self)
        self.host = host
        self.port = port
        self.protocal = HttpProtocal(host, port, timeout=timeout)
        self.servers = servers
        self.timeout = timeout
        if len(self.servers) == 0 and self.host != '' and self.port != 0:
            self.servers = [ host+':'+str(port) ]
        for server in self.servers:
            host, port = server.split(':')
            self.serversProtocalMap[ server ] = HttpProtocal(host, int(port), timeout=timeout)

    @retryTimes(retryTimes=3)
    def call(self, func, args = ()):
        self.beforeCall()
        package = {
            'func' : func,
            'args' : args,
        }
        msg = self.protocal.serialize(package)
        respmsg = self.protocal.transport.send(msg)
        resp = self.protocal.unserialize(respmsg)
        if isinstance(resp, FuncNotFoundException):
            raise resp
        return resp

    def getProtocalInstance(self, instance):
        key = "%s:%s:%s" % (instance.service, instance.address, instance.port)
        if key in self.protocalMap:
            return self.protocalMap.get(key)
        self.protocalMap[key] =  HttpProtocal(instance.address, instance.port)
        return self.protocalMap[key]


#coding=utf-8

import time


class MemValue:
    def __init__(self, value='', ttl=-1):
        self.value = value
        self.createStamp = time.time()
        self.ttl = ttl

    def isExpire(self):
        if self.ttl == -1:
            return False
        now = time.time()
        diff = now - self.createStamp
        if diff >= self.ttl:
            return True
        return False


class MemStringCache:

    instance = None

    def __init__(self):
        self.data = {}

    @classmethod
    def getInstance(cls):
        if cls.instance == None:
            cls.instance = MemStringCache()
        return cls.instance

    def set(self, k, v, ttl=-1):
        self.data[k] = MemValue(value=v, ttl=ttl)

    def get(self, k):
        if k not in self.data:
            return None
        memVal = self.data[k]
        if memVal.isExpire():
            del self.data[k]
            return None
        return memVal.value

    def count(self):
        cnt = 0
        for k in self.data.keys():
            v = self.get(k)
            if v == None:
                continue
            cnt = cnt + 1
        return cnt



#coding=utf-8

import datetime
import time


def today():
    return datetime.date.today()


def lastday():
    return str(today() - datetime.timedelta(1))


def tommorrow():
    return today() + datetime.timedelta(1)


def stamp_to_date_str(stamp):
    return time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(int(stamp)))


def date_str_to_stamp(dateString):
    dateString = str(dateString)
    return int(time.mktime(time.strptime(dateString, '%Y-%m-%d %H:%M:%S')))


def current_time():
    return str(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time())))


def datestr_to_zh_time(date_str):
    '''
    如果时间格式是一天之内的，原样返回
    否则返回1 天前，2 天前，3 天前 这种
    '''
    date_str = str(date_str)
    stamp = date_str_to_stamp(date_str)
    lastdaystr = lastday() + ' 23:59:59'
    lastdaystamp = date_str_to_stamp(lastdaystr)
    if stamp > lastdaystamp:
        return date_str
    else:
        diff = lastdaystamp - stamp
        days = diff / (3600 * 24) + 1
        return str(days) + ' 天前'



#coding=utf-8

import hashlib
import json
import requests
from agileutil.util import local_ip
import time


class Instance(object):

    def __init__(self, service, address, port):
        self.service = service
        self.address = address
        self.port = port


class ConsulApi(object):

    def __init__(self, host: str, port: int):
        self.host = host
        self.port = port
        self.baseUrl = 'http://%s:%s' % (self.host, self.port)
        self.token = None
        self.timeout = 5

    def put(self, k: str, v: str):
        url = self.baseUrl + '/v1/kv/%s' % k        
        r = requests.put(url, data=v, timeout = self.timeout)
        if r.status_code != 200:
            raise Exception(r.text)

    def getInstanceMap(self):
        url = self.baseUrl + '/v1/agent/services'
        r = requests.get(url, timeout = self.timeout)
        if r.status_code != 200:
            raise Exception(r.text)
        #print(r.text)
        return json.loads(r.text)

    def genInstanceID(self, service: str, host: str, port: int):
        key = "%s:%s:%s" % (service, host, port)
        m = hashlib.md5()
        m.update(key.encode())
        return m.hexdigest()

    def registService(self, serviceName: str, port = 80, address=None, ttl=30, degisterAfter = '1m'):
        params = {
            'Name' : serviceName,
            'Port' : port
        }
        if address:
            params['Address'] = address
        else:
            params['Address'] = local_ip()
        params['ID'] = self.genInstanceID(serviceName, params['Address'], port)
        '''
        params['Check'] = {
            'Interval' : "%ss" % checkInterval,
            'DeregisterCriticalServiceAfter' : '1m',
            'Timeout' : '5s',
            'TCP' : "%s:%s" % (params['Address'], port)
        }
        '''
        params['Check'] = {
            'DeregisterCriticalServiceAfter' : degisterAfter,
            'TTL' : '%ss' % ttl,
        }
        #print('params', json.dumps(params))
        url = self.baseUrl + '/v1/agent/service/register'
        r = requests.put(url, data=json.dumps(params), timeout = self.timeout)
        print(r.text, r.status_code)


    def deregistService(self, serviceName: str, port: int, address = None):
        instanceId = self.genInstanceID(serviceName, address, port)
        url = self.baseUrl + '/v1/agent/service/deregister/%s' % instanceId
        r = requests.put(url, timeout = self.timeout)
        if r.status_code != 200:
            raise Exception(r.text)

    def get(self, k: str):
        url = self.baseUrl + '/v1/kv/%s?raw=true' % k  
        r = requests.get(url, timeout = self.timeout)
        if r.status_code != 200:
            raise Exception(r.text)
        return r.text

    def getServiceInstanceList(self, serviceName: str) -> list:
        instanceMap = self.getInstanceMap()
        instanceList = []
        for instanceID, info in instanceMap.items():
            if info.get('Service') == serviceName:
                instance = Instance(
                    service = serviceName,
                    address = info.get('Address'),
                    port = info.get('Port')
                )
                instanceList.append(instance)
        return instanceList

    def ttlHeartbeat(self, service, address, port):
        url = self.baseUrl + '/v1/agent/check/pass/%s' % ('service:' + self.genInstanceID(service, address, port))
        r = requests.put(url, timeout = self.timeout)
        if r.status_code != 200:
            raise Exception(r.text)


import os
import socket
import commands
import time
import sys

RECIEVER = 'xxx@xxxx.cn'


def sendmail(sender, reciever, title, content):
    cmd = "echo '%s' | mail -s '%s' -r '%s' '%s'" % (content, title, sender,
                                                     reciever)
    commands.getstatusoutput(cmd)


def notify(user, action):
    if user.strip() == '': return
    hostname = socket.gethostname()
    curTime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
    sendmail(
        'ssh_notify@' + hostname,
        RECIEVER,
        'user %s %s %s at %s' % (user, action, hostname, curTime),
        'user %s %s %s at %s' % (user, action, hostname, curTime),
    )


pid = os.fork()
if pid != 0: sys.exit(0)

users = []
firstTag = True

while True:
    status, output = commands.getstatusoutput('users')
    curUsers = list(set(output.split(' ')))
    if 'root' in curUsers: curUsers.remove('root')
    for user in curUsers:
        if user not in users:
            users.append(user)
            if not firstTag: notify(user, 'login')
    for user in users:
        if user not in curUsers:
            users.remove(user)
            notify(user, 'logout')
    firstTag = False
    time.sleep(5)



#coding=utf-8
'''
required ansible version <= 2.0
'''

import ansible.runner
import json


def run(module_name,
        module_args='',
        forks=1,
        host_list=[],
        ssh_user='',
        ssh_pass=''):
    runner = ansible.runner.Runner(module_name=module_name,
                                   module_args=module_args,
                                   forks=forks,
                                   host_list=host_list,
                                   extra_vars={
                                       "ansible_ssh_user": ssh_user,
                                       "ansible_ssh_pass": ssh_pass
                                   })
    datastructure = runner.run()
    return json.dumps(datastructure)



import web
import pymysql
import json
import time
from webpy_base import WebPyBase

#coding=utf-8
from sanic import Sanic
from sanic.response import json, text


class SanicBase:
    def GET(self):
        return text(self.handle())

    def POST(self):
        return text(self.handle())

    def handle(self):
        return 'ok'

    def req(self, k):
        pass


class Test(SanicBase):
    def handle(self):
        print('new req')
        time.sleep(60)
        return 'ok'


urls = ('/intf/test', SanicBase)

if __name__ == '__main__':
    app = Sanic('')
    #for uri, action in urls.items(): app.add_route(action.as_view(), uri)
    for item in [urls[i:i + 2] for i in range(0, len(urls), 2)]:
        app.add_route(item[1].as_view(), item[0])
    app.run(host='0.0.0.0', port=8000, debug=True)
    #web.application(urls, globals()).run()



#coding=utf-8
'''
pip install pexpect
'''

import pexpect


class SSHClient(object):

    __slots__ = ['_host', '_port', '_user', '_pwd', '_ssh']

    def __init__(self, host, port, user, pwd):
        self._host = host
        self._port = port
        self._user = user
        self._pwd = pwd
        self._ssh = None

    def run(self, cmd):
        ssh_newkey = 'Are you sure you want to continue connecting'
        child = pexpect.spawn('ssh -l %s -p %s %s %s' %
                              (self._user, self._port, self._host, cmd))
        i = child.expect([pexpect.TIMEOUT, ssh_newkey, 'password: '])
        if i == 0:  #timeout
            raise Exception('ssh timeout')
        if i == 1:  #SSH does not have the public key. Just accept it.
            child.sendline('yes')
            child.expect('password: ')
            i = child.expect([pexpect.TIMEOUT, 'password: '])
            if i == 0:  #timeout
                raise Exception('ssh timeout')
        child.sendline(self._pwd)
        child.expect(pexpect.EOF)
        return child.before

    def getAddr(self):
        return self._host + ':' + str(self._port)


def ssh_exec(host, port, user, pwd, cmd):
    client = SSHClient(host, port, user, pwd)
    return client.run(cmd)


def batch_ssh_exec(host_list, user, pwd, cmd, port=22):
    for host in host_list:
        print(ssh_exec(host, port, user, pwd, cmd))



#coding=utf-8
'''
压力测试时使用
locust -f locust_demo.py
'''

from locust import HttpLocust, TaskSet, task, between


class WebsiteTasks(TaskSet):
    @task
    def test(self):
        self.client.get("/hello")


class WebsiteUser(HttpLocust):
    host = "http://144.34.213.205:10001"  #被测系统的host
    task_set = WebsiteTasks  #该类定义用户任务信息，该类继承TaskSet
    wait_time = between(1000, 2000)
    stop_timeout = 30



#coding=utf-8
'''
producer:
    rbtctl = RabbitmqCtl(
        host = 'xxxxxxxx',
        port = 5672,
        user = 'admin',
        pwd = 'admin',
        vhost = 'common',
        exchange = 'ha_exchange',
        queue = 'test_que',
        routing_key = 'default'
    )
    rbtctl.push('aaaa')
    print rbtctl.pull()
'''

import pika

MSG_PERSISTENT = 2


class RabbitmqCtl:
    def __init__(self,
                 host,
                 port,
                 user,
                 pwd,
                 vhost,
                 exchange,
                 queue,
                 routing_key,
                 durable=True,
                 exchange_type='direct',
                 prefetch_count=1):
        self._host = host
        self._port = port
        self._user = user
        self._pwd = pwd
        self._vhost = vhost
        self._exchange = exchange
        self._queue = queue
        self._routing_key = routing_key
        self._durable = durable
        self._exchange_type = exchange_type
        self._prefetch_count = prefetch_count
        self._credentials = None
        self._params = None
        self._connection = None
        self._channel = None

    def connect(self):
        if self._connection is None:
            self._credentials = pika.PlainCredentials(self._user, self._pwd)
            self._params = pika.ConnectionParameters(self._host, self._port,
                                                     str(self._vhost),
                                                     self._credentials)
            self._connection = pika.BlockingConnection(self._params)
            self._channel = self._connection.channel()
            self._channel.exchange_declare(durable=self._durable,
                                           exchange=self._exchange,
                                           exchange_type=self._exchange_type)
            self._channel.queue_declare(queue=self._queue,
                                        durable=self._durable)
            self._channel.basic_qos(prefetch_count=1)

    def close(self):
        if self._connection:
            self._connection.close()
            self._credentials = None
            self._params = None
            self._connection = None
            self._channel = None

    def push(self, msgBody):
        if self._connection is None: self.connect()

        self._channel.basic_publish(
            exchange=self._exchange,
            routing_key=self._routing_key,
            body=msgBody,
            properties=pika.BasicProperties(delivery_mode=MSG_PERSISTENT))

    def pull(self, no_ack_tag=True):
        if self._connection is None: self.connect()
        method, properties, body = self._channel.basic_get(queue=self._queue,
                                                           no_ack=no_ack_tag)
        return body



#coding=utf-8

import mail


def send(content='', title='agileutil_alarm', mail_to='lycclsltt@163.com'):
    try:
        mail.send(from_addr='agileutil_alarm@agileutil.com',
                  to_addr=mail_to,
                  title=title,
                  content=content)
    except:
        pass



#coding=utf-8

import smtplib
from email.mime.text import MIMEText
from email.header import Header
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
import socket
import traceback


class MailServerConfig(object):

    def __init__(self, mailHost, mailPort, mailUser, mailPass, timeout = 30):
        self.mailHost = mailHost
        self.mailPort = mailPort
        self.mailUser = mailUser
        self.mailPass = mailPass
        self.timeout = timeout


class MailMeta(object):

    def __init__(self, sender, receiver, subject, html, pic = None):
        self.sender = sender
        self.receiver = receiver
        self.subject = subject
        self.html = html
        self.pic = pic
        self.message = None

    def getMessage(self):
        if self.message == None:
            message = MIMEMultipart()
            message['Subject'] = Header(self.subject, 'utf-8').encode()
            message['From'] = Header(self.sender, 'utf-8').encode()
            if self.pic:
                f = open(self.pic, 'rb')
                msgImage = MIMEImage(f.read())
                f.close()
                msgImage.add_header('Content-ID', 'pic')
                msgImage["Content-Disposition"] = 'attachment; filename="pic.png"'
                message.attach(msgImage)
            message.attach(MIMEText(self.html, 'html', 'utf-8'))
            self.message = message
        return self.message.as_string()



class MailSender(object):

    def __init__(self, mailServerConf):
        self.mailServerConfig = mailServerConf

    def send(self, mail):
        '''
        :param mail: MailMeta obj
        :return: 成功返回None, 否则返回错误信息
        '''
        try:
            socket.setdefaulttimeout(self.mailServerConfig.timeout)
            smtpObj = smtplib.SMTP(host=self.mailServerConfig.mailHost, port=self.mailServerConfig.mailPort)
            smtpObj.ehlo()  #发送SMTP 'ehlo' 命令
            smtpObj.starttls()
            smtpObj.login(self.mailServerConfig.mailUser, self.mailServerConfig.mailPass)
            smtpObj.sendmail(from_addr=mail.sender, to_addrs=mail.receiver, msg=mail.getMessage())
            smtpObj.quit()
            return None
        except Exception as ex:
            return str(ex) + traceback.format_exc()


#coding=utf-8

import ujson
import web
from web.contrib.template import render_jinja
import sys
try:
    from imp import reload
except:
    pass
reload(sys)
try:
    sys.setdefaultencoding('utf-8')
except:
    pass
import platform


class WebPyBase:
    def render(self, page_dir='static/pages/', encode='utf-8'):
        rend = render_jinja(page_dir, encoding=encode)
        return rend

    def GET(self):
        return self.handle()

    def POST(self):
        return self.handle()

    def handle(self):
        return ''

    def request(self, k):
        i = web.input()
        v = ''
        try:
            v = i[k]
        except:
            pass
        return v.strip()

    def req(self, k):
        return self.request(k)

    def resp(self,
             errno=0,
             errmsg='',
             data='',
             page='',
             pagesize='',
             control_allow='*'):
        try:
            web.header("Access-Control-Allow-Origin", control_allow)
        except:
            pass
        ret = {}
        if platform.python_version_tuple()[0] == '2':
            ret = {'errno': errno, 'errmsg': unicode(errmsg), 'data': data}
        else:
            ret = {'errno': errno, 'errmsg': errmsg, 'data': data}
        if page != '' and pagesize != '':
            ret['page'] = page
            ret['pagesize'] = pagesize
        return ujson.dumps(ret)

    def session(self, k, v=None):
        '''
        init in main application:
        
        def session_hook():
            web.ctx.session = session
        app.add_processor(web.loadhook(session_hook))
        
        '''
        if v == None:
            #get
            v = ''
            try:
                v = web.ctx.session[k]
            except Exception as ex:
                pass
            return v
        else:
            #set
            try:
                web.ctx.session[k] = v
                #print('[session] set %s to %s' % (str(k), str(v)))
            except Exception as ex:
                #print('[session] web.ctx.session[k] = v exception:', ex)

    def kill_session(self):
        web.ctx.session.kill()

    def deny(self, errmsg=u'您没有权限'):
        return self.resp(errno=1, errmsg=errmsg)



#coding=utf-8

import requests
import ujson


class K8sClient:

    STRAGE_ROLLING_UPDATE = 'RollingUpdate'
    STRAGE_RECREATE = 'Recreate'
    STRAGE_DEFAULT = 'RollingUpdate'

    PROTOCOL_TCP = 'TCP'
    PROTOCOL_UDP = 'UDP'
    PROTOCOL_SCTP = 'SCTP'

    def __init__(self, baseUrl, token='', timeout=10):
        if baseUrl[-1] == '/': baseUrl[-1] = ''
        self.baseUrl = baseUrl
        self.token = token
        self.headers = {
            'Authorization': 'Bearer ' + self.token,
            'Content-Type': 'application/json'
        }
        self.verify = False
        self.timeout = timeout

    def createNamespace(self, namespace):
        '''
        创建namespace
        参考:
        https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/#namespace-v1-core
        https://kubernetes.io/docs/tasks/administer-cluster/namespaces/
        '''
        url = self.baseUrl + '/api/v1/namespaces'
        body = {
            'apiVersion': "v1",
            'kind': 'Namespace',
            'metadata': {
                'name': namespace,
                'labels': {
                    'name': namespace,
                }
            }
        }
        r = requests.post(url,
                          headers=self.headers,
                          data=ujson.dumps(body),
                          verify=self.verify,
                          timeout=self.timeout)
        return r.status_code, r.text

    def deleteNamespace(self, namespace):
        '''
        删除namespace
        '''
        url = self.baseUrl + '/api/v1/namespaces/%s' % namespace
        r = requests.delete(url, headers=self.headers, verify=self.verify)
        return r.status_code, r.text

    def getNamespace(self, namespace):
        '''
        获取一个namespace
        '''
        url = self.baseUrl + '/api/v1/namespaces/%s' % namespace
        r = requests.get(url,
                         headers=self.headers,
                         verify=self.verify,
                         timeout=self.timeout)
        return r.status_code, r.text

    def listNamespace(self):
        '''
        获取所有namespace
        '''
        url = self.baseUrl + '/api/v1/namespaces'
        r = requests.get(url,
                         headers=self.headers,
                         verify=self.verify,
                         timeout=self.timeout)
        return r.status_code, r.text

    def getPods(self, namespace='default', labelSelector={}):
        #获取某个namespace下的pod列表，默认选择default的
        url = self.baseUrl + '/api/v1/namespaces/%s/pods/' % namespace
        if type(labelSelector) == dict and len(labelSelector) != 0:
            queryList = []
            for k, v in labelSelector.items():
                queryList.append('%s=%s' % (k, v))
            queryString = ','.join(queryList)
            url = url + '?labelSelector=' + queryString
        r = requests.get(url,
                         headers=self.headers,
                         verify=self.verify,
                         timeout=self.timeout)
        return r.status_code, r.text

    @classmethod
    def makeDeploymentBody(cls,
                           deployment,
                           env,
                           image='ubuntu:16.04',
                           portList=[],
                           namespace='default',
                           replicas=0,
                           minReadySeconds=0,
                           paused=False,
                           progressDeadlineSeconds=600,
                           revisionHistoryLimit=100,
                           strategy=STRAGE_DEFAULT):
        body = {
            'apiVersion': 'apps/v1',
            'kind': 'Deployment',
            'metadata': {
                'name': deployment,
                'namespace': namespace,
            },
            'spec': {
                'minReadySeconds': minReadySeconds,
                'paused': paused,
                'progressDeadlineSeconds': progressDeadlineSeconds,
                'replicas': replicas,
                'revisionHistoryLimit': revisionHistoryLimit,
                'selector': {
                    'matchLabels': {
                        'app': deployment,  #这里和服务名称一致
                        'env': env,  #这里和环境名称一致(sep, rc, release, prod)
                    },
                },
                'strategy': {
                    'type': strategy
                },
                'template': {
                    'metadata': {
                        'labels': {
                            'app': deployment,
                            'env': env
                        }
                    },
                    'spec': {
                        'containers': [{
                            'name': deployment,
                            'image': image,
                            'ports': portList
                        }]
                    }
                }
            }
        }

        return body

    def createDeployment(self,
                         deployment,
                         env,
                         image='ubuntu:16.04',
                         portList=[],
                         namespace='default',
                         replicas=0,
                         minReadySeconds=0,
                         paused=False,
                         progressDeadlineSeconds=600,
                         revisionHistoryLimit=100,
                         strategy=STRAGE_DEFAULT):
        '''
        创建deployment
        参考：
        https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.13/#deployment-v1-apps

        参数:
        namespace: 创建属于哪个namespace的deployment
        deployment: deployment名称, 传递时和容器服务名称一致,会被存放到matchlabels中
        replicas: 副本数，默认为0
        minReadySeconds: 可用状态的最小秒数。默认是0（Pod在ready后就会被认为是可用状态
        paused: boolean值, 用来指定暂停和恢复Deployment, 默认False
        progressDeadlineSeconds: 表示Deployment controller等待多少秒才能确定（通过 Deployment status）Deployment进程是卡住的。根据k8s官方，默认为600s
        revisionHistoryLimit: kubectl apply 每次更新应用时 Kubernetes 都会记录下当前的配置，保存为一个 revision（版次），这样就可以回滚到某个特定 revision。默认配置下，Kubernetes 只会保留最近的几个 revision，可以在 Deployment 配置文件中通过 revisionHistoryLimit 属性增加 revision 数量。这里默认设为100.
        strategy: 指定新的Pod替换旧的Pod的策略, Recreate 或 RollingUpdate, 默认为RollingUpdate.Recreate时，在创建出新的Pod之前会先杀掉所有已存在的Pod.
        image: pod运行的镜像
        portList:[ {'name' : 'xxx', 'protocol' : 'TCP/UDP/SCTP', 'hostPort':80, 'hostIP':'0.0.0.0', 'containerPort':80} ] 端口列表

        env:属于哪个环境, sep, rc, release, prod, 会被存放到matchlabels中

        返回:
        状态码，输出, map, json string
        '''
        url = self.baseUrl + '/apis/apps/v1/namespaces/%s/deployments' % namespace

        body = K8sClient.makeDeploymentBody(deployment, env, image, portList,
                                            namespace, replicas,
                                            minReadySeconds, paused,
                                            progressDeadlineSeconds,
                                            revisionHistoryLimit, strategy)

        r = requests.post(url,
                          headers=self.headers,
                          data=ujson.dumps(body),
                          verify=self.verify,
                          timeout=self.timeout)
        return r.status_code, r.text, body, ujson.dumps(body)

    @classmethod
    def makePodBody(cls,
                    podName,
                    image,
                    namespace='default',
                    labels={},
                    annotations={},
                    nodeName='',
                    containerName='',
                    ports=[],
                    command=[]):
        body = {
            'apiVersion': "v1",
            'kind': 'Pod',
            'metadata': {
                'name': podName,
                'namespace': namespace,
                'labels': labels,
                'annotations': annotations,
            },
            'spec': {
                'containers': [{
                    'name': containerName,
                    'image': image,
                    'ports': ports,
                    'command': command,
                }]
            }
        }

        if nodeName != '':
            body['spec']['nodeName'] = nodeName

        if containerName == '':
            body['spec']['containers'][0]['name'] = podName

        return body

    def createPod(self,
                  podName,
                  image,
                  namespace='default',
                  labels={},
                  annotations={},
                  nodeName='',
                  containerName='',
                  ports=[],
                  command=[]):
        url = self.baseUrl + '/api/v1/namespaces/%s/pods' % namespace

        body = K8sClient.makePodBody(podName, image, namespace, labels,
                                     annotations, nodeName, containerName,
                                     ports, command)

        r = requests.post(url,
                          headers=self.headers,
                          data=ujson.dumps(body),
                          verify=self.verify,
                          timeout=self.timeout)
        return r.status_code, r.text, body, ujson.dumps(body)

    def deletePod(self, podName, namespace='default'):
        url = self.baseUrl + '/api/v1/namespaces/%s/pods/%s' % (namespace,
                                                                podName)
        r = requests.delete(url,
                            headers=self.headers,
                            verify=self.verify,
                            timeout=self.timeout)
        return r.status_code, r.text

    def replacePod(self, podName, podBody, namesapce='default'):
        url = self.baseUrl + '/api/v1/namespaces/%s/pods/%s' % (namesapce,
                                                                podName)
        r = requests.put(url,
                         data=podBody,
                         headers=self.headers,
                         verify=self.verify,
                         timeout=self.timeout)
        return r.status_code, r.text

    def getPod(self, podName, namespace='default'):
        url = self.baseUrl + '/api/v1/namespaces/%s/pods/%s' % (namespace,
                                                                podName)
        r = requests.get(url,
                         headers=self.headers,
                         verify=self.verify,
                         timeout=self.timeout)
        return r.status_code, r.text

    def getPodStatus(self, pod):
        '''
        成功返回状态字符串，None
        失败返回'', 错误信息
        '''
        status = pod['status']
        ret = ''

        print(status)

        try:
            if pod['status']['containerStatuses'][0]['ready'] == True:
                ret = pod['status']['phase']
            else:
                stateMap = pod['status']['containerStatuses'][0]['state']
                for state, desc in stateMap.items():
                    ret = desc['reason']
                    break
        except Exception as ex:
            return ret, str(ex)
        return ret, None


'''
#get pod status
token = """eyJhbGciOiJSUzI1NiIsImtpZCI6IiJ9.eyJpc3MiOiJrdWJlcm5ldGVzL3NlcnZpY2VhY2NvdW50Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9uYW1lc3BhY2UiOiJrdWJlLXN5c3RlbSIsImt1YmVybmV0ZXMuaW8vc2VydmljZWFjY291bnQvc2VjcmV0Lm5hbWUiOiJlY2RhdGFhcGktdG9rZW4tdmZyY24iLCJrdWJlcm5ldGVzLmlvL3NlcnZpY2VhY2NvdW50L3NlcnZpY2UtYWNjb3VudC5uYW1lIjoiZWNkYXRhYXBpIiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQudWlkIjoiNzFlOWViOTMtZTYzYy0xMWU4LTk5ZTUtNzgyYmNiNTkwNTcwIiwic3ViIjoic3lzdGVtOnNlcnZpY2VhY2NvdW50Omt1YmUtc3lzdGVtOmVjZGF0YWFwaSJ9.Q_M4WLYmMWEUGcNwdt6fRpIWGhjAGsabXUWOzthuluUBb_jgKFDF4P0bvqq2gHe0mVePw-wrtCtnHIXEhHptT_SskstGb3AlyePlh0_4ixsVoS6R10ksnEbk_BdC1AU18Yq3A4MQ2f8-NivKddok8HyTRN-wXe5Ilw7ion7ZeU7RFvk-PSqV_QsnFThdiyPy1cGeDj0VWBu9ceSExsR5L879w6YwJMbqUFePYsxqSVjrJ2o7WrqLDb_fdlQGG50qFOYdVj9iTXj_mCEQHw9Xxv6KNWTDaIYpFxrrItYnnLT3wAYp0z_K2Qi4ruiQpc14-snGOD1FgJGh6IzUN_lSKg"""
cli = K8sClient(baseUrl = 'https://10.12.35.3:6443', token = token)
code, resp = cli.getPod('pod-10-20-0-130', 'sce')
pod = ujson.loads(resp)
status, err = cli.getPodStatus(pod)
print('status:', status)
print('err:', err)
'''
'''
{
	'phase': 'Running',
	'conditions': [{
		'type': 'Initialized',
		'status': 'True',
		'lastProbeTime': None,
		'lastTransitionTime': '2018-12-28T07:47:20Z'
	}, {
		'type': 'Ready',
		'status': 'True',
		'lastProbeTime': None,
		'lastTransitionTime': '2018-12-28T07:47:23Z'
	}, {
		'type': 'ContainersReady',
		'status': 'True',
		'lastProbeTime': None,
		'lastTransitionTime': '2018-12-28T07:47:23Z'
	}, {
		'type': 'PodScheduled',
		'status': 'True',
		'lastProbeTime': None,
		'lastTransitionTime': '2018-12-28T07:47:20Z'
	}],
	'hostIP': '10.12.35.2',
	'podIP': '10.20.0.130',
	'startTime': '2018-12-28T07:47:20Z',
	'containerStatuses': [{
		'name': 'container-10-20-0-130',
		'state': {
			'running': {
				'startedAt': '2018-12-28T07:47:22Z'
			}
		},
		'lastState': {},
		'ready': True,
		'restartCount': 0,
		'image': '10.12.35.2/sce/sce-go-test:go-9ba6ba10-prod-1545892604',
		'imageID': 'docker-pullable://10.12.35.2/sce/sce-go-test@sha256:d148ed2775692453da324ca7662b9af87319d126ef92a2d040a6d9727263392f',
		'containerID': 'docker://056dfa639d8806089bcfee1b9ca6a4fd5b97ebb1e023b338b435cc7e1f0f8c35'
	}],
	'qosClass': 'BestEffort'
}
'''
'''
token = """eyJhbGciOiJSUzI1NiIsImtpZCI6IiJ9.eyJpc3MiOiJrdWJlcm5ldGVzL3NlcnZpY2VhY2NvdW50Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9uYW1lc3BhY2UiOiJrdWJlLXN5c3RlbSIsImt1YmVybmV0ZXMuaW8vc2VydmljZWFjY291bnQvc2VjcmV0Lm5hbWUiOiJlY2RhdGFhcGktdG9rZW4tdmZyY24iLCJrdWJlcm5ldGVzLmlvL3NlcnZpY2VhY2NvdW50L3NlcnZpY2UtYWNjb3VudC5uYW1lIjoiZWNkYXRhYXBpIiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQudWlkIjoiNzFlOWViOTMtZTYzYy0xMWU4LTk5ZTUtNzgyYmNiNTkwNTcwIiwic3ViIjoic3lzdGVtOnNlcnZpY2VhY2NvdW50Omt1YmUtc3lzdGVtOmVjZGF0YWFwaSJ9.Q_M4WLYmMWEUGcNwdt6fRpIWGhjAGsabXUWOzthuluUBb_jgKFDF4P0bvqq2gHe0mVePw-wrtCtnHIXEhHptT_SskstGb3AlyePlh0_4ixsVoS6R10ksnEbk_BdC1AU18Yq3A4MQ2f8-NivKddok8HyTRN-wXe5Ilw7ion7ZeU7RFvk-PSqV_QsnFThdiyPy1cGeDj0VWBu9ceSExsR5L879w6YwJMbqUFePYsxqSVjrJ2o7WrqLDb_fdlQGG50qFOYdVj9iTXj_mCEQHw9Xxv6KNWTDaIYpFxrrItYnnLT3wAYp0z_K2Qi4ruiQpc14-snGOD1FgJGh6IzUN_lSKg"""
cli = K8sClient(baseUrl = 'https://10.12.35.3:6443', token = token)
code, resp = cli.getPods(namespace='sce', labelSelector={'app':'sce-java-test', 'env' : 'prod'})
print(code)
print(resp)
'''
'''
#get pod
token = """eyJhbGciOiJSUzI1NiIsImtpZCI6IiJ9.eyJpc3MiOiJrdWJlcm5ldGVzL3NlcnZpY2VhY2NvdW50Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9uYW1lc3BhY2UiOiJrdWJlLXN5c3RlbSIsImt1YmVybmV0ZXMuaW8vc2VydmljZWFjY291bnQvc2VjcmV0Lm5hbWUiOiJlY2RhdGFhcGktdG9rZW4tdmZyY24iLCJrdWJlcm5ldGVzLmlvL3NlcnZpY2VhY2NvdW50L3NlcnZpY2UtYWNjb3VudC5uYW1lIjoiZWNkYXRhYXBpIiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQudWlkIjoiNzFlOWViOTMtZTYzYy0xMWU4LTk5ZTUtNzgyYmNiNTkwNTcwIiwic3ViIjoic3lzdGVtOnNlcnZpY2VhY2NvdW50Omt1YmUtc3lzdGVtOmVjZGF0YWFwaSJ9.Q_M4WLYmMWEUGcNwdt6fRpIWGhjAGsabXUWOzthuluUBb_jgKFDF4P0bvqq2gHe0mVePw-wrtCtnHIXEhHptT_SskstGb3AlyePlh0_4ixsVoS6R10ksnEbk_BdC1AU18Yq3A4MQ2f8-NivKddok8HyTRN-wXe5Ilw7ion7ZeU7RFvk-PSqV_QsnFThdiyPy1cGeDj0VWBu9ceSExsR5L879w6YwJMbqUFePYsxqSVjrJ2o7WrqLDb_fdlQGG50qFOYdVj9iTXj_mCEQHw9Xxv6KNWTDaIYpFxrrItYnnLT3wAYp0z_K2Qi4ruiQpc14-snGOD1FgJGh6IzUN_lSKg"""
cli = K8sClient(baseUrl = 'https://10.12.35.3:6443', token = token)
code, resp = cli.getPod('lyc-test-pod-16', 'sce')
print(code, resp)
'''
'''
#replace pod
podBody = K8sClient.makePodBody(
    podName = 'lyc-test-pod-12', 
    image = 'nginx:1.7.9', 
    namespace='sce',
    labels = {
        'env' : 'sep',
        'app' : app,
    },
    annotations = {
        'cni.projectcalico.org/ipAddrs' : "[\"10.20.35.8\"]"
    },
    nodeName = 'bjxg-ap-35-2',
    containerName = 'lyc-test-container-12',
    ports = [
        {
            'containerPort' : 81,
        }
    ]
)

podBody = """{"apiVersion":"v1","kind":"Pod","metadata":{"annotations":{"cni.projectcalico.org/ipAddrs":"['10.20.35.9']"},"labels":{"app":"lyctest","env":"sep"},"name":"lyc-test-pod-13"},"spec":{"containers":[{"command":[],"image":"nginx:1.7.9","name":"lyc-test-container-13","ports":[{"containerPort":81}]}],"nodeName":"bjxg-ap-35-2"}}"""
code, resp = cli.replacePod(
    podName = 'lyc-test-pod-13',
    podBody = podBody,
    namesapce= 'sce'
)
print(podBody)
print(code)
print(resp)
'''
'''
#delete pod
token = """eyJhbGciOiJSUzI1NiIsImtpZCI6IiJ9.eyJpc3MiOiJrdWJlcm5ldGVzL3NlcnZpY2VhY2NvdW50Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9uYW1lc3BhY2UiOiJrdWJlLXN5c3RlbSIsImt1YmVybmV0ZXMuaW8vc2VydmljZWFjY291bnQvc2VjcmV0Lm5hbWUiOiJlY2RhdGFhcGktdG9rZW4tdmZyY24iLCJrdWJlcm5ldGVzLmlvL3NlcnZpY2VhY2NvdW50L3NlcnZpY2UtYWNjb3VudC5uYW1lIjoiZWNkYXRhYXBpIiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQudWlkIjoiNzFlOWViOTMtZTYzYy0xMWU4LTk5ZTUtNzgyYmNiNTkwNTcwIiwic3ViIjoic3lzdGVtOnNlcnZpY2VhY2NvdW50Omt1YmUtc3lzdGVtOmVjZGF0YWFwaSJ9.Q_M4WLYmMWEUGcNwdt6fRpIWGhjAGsabXUWOzthuluUBb_jgKFDF4P0bvqq2gHe0mVePw-wrtCtnHIXEhHptT_SskstGb3AlyePlh0_4ixsVoS6R10ksnEbk_BdC1AU18Yq3A4MQ2f8-NivKddok8HyTRN-wXe5Ilw7ion7ZeU7RFvk-PSqV_QsnFThdiyPy1cGeDj0VWBu9ceSExsR5L879w6YwJMbqUFePYsxqSVjrJ2o7WrqLDb_fdlQGG50qFOYdVj9iTXj_mCEQHw9Xxv6KNWTDaIYpFxrrItYnnLT3wAYp0z_K2Qi4ruiQpc14-snGOD1FgJGh6IzUN_lSKg"""
cli = K8sClient(baseUrl = 'https://10.12.35.3:6443', token = token)
code, resp = cli.deletePod('lyc-test-pod-11', 'sce')
print(code, resp)
'''
'''
#create pod
token = """eyJhbGciOiJSUzI1NiIsImtpZCI6IiJ9.eyJpc3MiOiJrdWJlcm5ldGVzL3NlcnZpY2VhY2NvdW50Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9uYW1lc3BhY2UiOiJrdWJlLXN5c3RlbSIsImt1YmVybmV0ZXMuaW8vc2VydmljZWFjY291bnQvc2VjcmV0Lm5hbWUiOiJlY2RhdGFhcGktdG9rZW4tdmZyY24iLCJrdWJlcm5ldGVzLmlvL3NlcnZpY2VhY2NvdW50L3NlcnZpY2UtYWNjb3VudC5uYW1lIjoiZWNkYXRhYXBpIiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQudWlkIjoiNzFlOWViOTMtZTYzYy0xMWU4LTk5ZTUtNzgyYmNiNTkwNTcwIiwic3ViIjoic3lzdGVtOnNlcnZpY2VhY2NvdW50Omt1YmUtc3lzdGVtOmVjZGF0YWFwaSJ9.Q_M4WLYmMWEUGcNwdt6fRpIWGhjAGsabXUWOzthuluUBb_jgKFDF4P0bvqq2gHe0mVePw-wrtCtnHIXEhHptT_SskstGb3AlyePlh0_4ixsVoS6R10ksnEbk_BdC1AU18Yq3A4MQ2f8-NivKddok8HyTRN-wXe5Ilw7ion7ZeU7RFvk-PSqV_QsnFThdiyPy1cGeDj0VWBu9ceSExsR5L879w6YwJMbqUFePYsxqSVjrJ2o7WrqLDb_fdlQGG50qFOYdVj9iTXj_mCEQHw9Xxv6KNWTDaIYpFxrrItYnnLT3wAYp0z_K2Qi4ruiQpc14-snGOD1FgJGh6IzUN_lSKg"""
cli = K8sClient(baseUrl = 'https://10.12.35.3:6443', token = token)
'''
'''
app = 'lyctest'
code, resp, body, json = cli.createPod(
    podName = 'lyc-test-pod-15', 
    image = 'nginx:1.7.9', 
    namespace='sce',
    labels = {
        'env' : 'sep',
        'app' : app,
    },
    annotations = {
        'cni.projectcalico.org/ipAddrs' : "[\"10.20.35.11\"]"
    },
    nodeName = 'bjxg-ap-35-2',
    containerName = 'lyc-test-container-15',
    ports = [
        {
            'containerPort' : 81,
        }
    ]
)
print(code)
print(resp)
print(json)
'''
'''
#create deployment
token = """eyJhbGciOiJSUzI1NiIsImtpZCI6IiJ9.eyJpc3MiOiJrdWJlcm5ldGVzL3NlcnZpY2VhY2NvdW50Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9uYW1lc3BhY2UiOiJrdWJlLXN5c3RlbSIsImt1YmVybmV0ZXMuaW8vc2VydmljZWFjY291bnQvc2VjcmV0Lm5hbWUiOiJlY2RhdGFhcGktdG9rZW4tdmZyY24iLCJrdWJlcm5ldGVzLmlvL3NlcnZpY2VhY2NvdW50L3NlcnZpY2UtYWNjb3VudC5uYW1lIjoiZWNkYXRhYXBpIiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZXJ2aWNlLWFjY291bnQudWlkIjoiNzFlOWViOTMtZTYzYy0xMWU4LTk5ZTUtNzgyYmNiNTkwNTcwIiwic3ViIjoic3lzdGVtOnNlcnZpY2VhY2NvdW50Omt1YmUtc3lzdGVtOmVjZGF0YWFwaSJ9.Q_M4WLYmMWEUGcNwdt6fRpIWGhjAGsabXUWOzthuluUBb_jgKFDF4P0bvqq2gHe0mVePw-wrtCtnHIXEhHptT_SskstGb3AlyePlh0_4ixsVoS6R10ksnEbk_BdC1AU18Yq3A4MQ2f8-NivKddok8HyTRN-wXe5Ilw7ion7ZeU7RFvk-PSqV_QsnFThdiyPy1cGeDj0VWBu9ceSExsR5L879w6YwJMbqUFePYsxqSVjrJ2o7WrqLDb_fdlQGG50qFOYdVj9iTXj_mCEQHw9Xxv6KNWTDaIYpFxrrItYnnLT3wAYp0z_K2Qi4ruiQpc14-snGOD1FgJGh6IzUN_lSKg"""
cli = K8sClient(baseUrl = 'https://10.12.35.3:6443', token = token)
code, resp, body, jsonStr = cli.createDeployment(
    namespace = 'sce',
    deployment = 'test-intf12',
    replicas = 2,
    env = 'prod',
    portList = [
        {
            'name' : 'testddrda-port1',
            'protocol' : K8sClient.PROTOCOL_TCP,
            'hostPort' : 80,
            'hostIP' : '0.0.0.0',
            'containerPort' : 80,
        }
    ]
)
print(code)
print(resp)
print('body:', body)
print('json str:', jsonStr)
'''



#coding=utf-8
'''
常用的装饰器
'''
from multiprocessing import Process
import functools
import time
import agileutil.date as dt
import socket


def stdout_log(func):
    '''
    打印方法耗时到标准输出
    例如: [2020-06-23 10:19:09] call test() method, args:(1, 2), return:3, cost:2.113 seconds
    '''
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        begin = time.time()
        ret = func(*args, **kwargs)
        cost = time.time() - begin
        #print('[%s] call %s() method, args:%s, return:%s, cost:%s seconds' %
        #      (dt.current_time(), func.__name__, str(args), str(ret),
        #       round(cost, 3)))
        return ret

    return wrapper


EXCEPTION = 'agileutil_wrap_exception'


def safe(func):
    '''
    对方法进行异常处理，异常不会被抛出, 减少导致进程crash的情况
    '''
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        ret = EXCEPTION
        try:
            ret = func(*args, **kwargs)
        except Exception as ex:
            pass
            #print(
            #    '[%s] catch exception when call %s() method, args:%s, ex:%s' %
            #    (dt.current_time(), func.__name__, str(args), str(ex)))
        return ret

    return wrapper


def retryTimes(retryTimes=2):
    '''
    如果装饰的方法抛出异常，那么进行重试n次，默认重试1次
    如果重试n次后仍然异常，那么抛出最后一个异常
    '''
    if retryTimes <= 0:
        retryTimes = 1
    
    def function(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            ret = None
            lastEx = None
            for i in range(retryTimes):
                try:
                    ret = func(*args, **kwargs)
                    return ret
                except socket.timeout as ex:
                    raise ex
                except Exception as ex:
                    lastEx = ex
                    #print(
                    #    '[%s] catch exception when call %s() method, args:%s, ex:%s, ready retry'
                    #    %
                    #    (dt.current_time(), func.__name__, str(args), str(ex)))
            raise lastEx

        return wrapper

    return function


def retry(func):
    '''
    如果装饰的方法抛出异常，那么进行重试1次
    如果重试1次后仍然异常，那么抛出最后一个异常
    '''
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        ret = None
        lastEx = None
        for i in range(2):
            try:
                ret = func(*args, **kwargs)
                return ret
            except socket.timeout as ex:
                raise ex
            except Exception as ex:
                lastEx = ex
                #print(
                #    '[%s] catch exception when call %s() method, args:%s, ex:%s, ready retry'
                #    % (dt.current_time(), func.__name__, str(args), str(ex)))
        raise lastEx

    return wrapper


def loop(func):
    '''
    当方法返回后，重新执行该方法
    '''
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        while 1:
            func(*args, **kwargs)

    return wrapper


def subprocess(func):
    '''
    在子进程中并发的执行此方法
    '''
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        p = Process(target=func, args=args, kwargs=kwargs)
        p.start()

    return wrapper


FUNC_EXEC_MAP = {}


def exec_once(func):
    '''
    被装饰的方法只会执行一次，之后即使显示调用，也不会被执行
    '''
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        global FUNC_EXEC_MAP
        if func.__name__ in FUNC_EXEC_MAP:
            return
        ret = func(*args, **kwargs)
        FUNC_EXEC_MAP[func.__name__] = 1
        return ret

    return wrapper


def force_return_string(func):
    '''
    被装饰的方法只会执行一次，之后即使显示调用，也不会被执行
    '''
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        ret = func(*args, **kwargs)
        return str(ret)

    return wrapper



# coding=utf-8

#如何查看key
#cat /home/user/.google_authenticator
#print(calGoogleCode(key))

import hmac, base64, struct, hashlib, time


# 参数secretKey是开通google身份验证时的密钥
def calGoogleCode(secretKey):
    input = int(time.time()) // 30
    key = base64.b32decode(secretKey)
    msg = struct.pack(">Q", input)
    googleCode = hmac.new(key, msg, hashlib.sha1).digest()
    o = ord(googleCode[19]) & 15
    googleCode = str(
        (struct.unpack(">I", googleCode[o:o + 4])[0] & 0x7fffffff) % 1000000)
    if len(googleCode) == 5:  # 如果验证码的第一位是0，则不会显示。此处判断若是5位码，则在第一位补上0
        googleCode = '0' + googleCode
    return googleCode



#coding=utf-8

from agileutil.rpc.protocal import TcpProtocal, UdpProtocal, HttpProtocal, RpcProtocal
import multiprocessing
from agileutil.rpc.exception import FuncNotFoundException
import queue
import threading
import select
import socket
import tornado.ioloop
from tornado.iostream import IOStream
from agileutil.rpc.discovery import DiscoveryConfig, ConsulRpcDiscovery
import struct
import functools
from agileutil.sanic import SanicController


class RpcServer(object):
    
    def __init__(self):
        self.funcMap = {}
        self.funcList = []
        self.discoveryConfig = None
        self.discovery = None
        self.protocal = RpcProtocal()

    def regist(self, func):
        self.funcMap[ func.__name__ ] = func
        self.funcList = self.funcMap.keys()

    def run(self, func, args):
        try:
            if func not in self.funcList:
                return FuncNotFoundException('func not found')
            funcobj = self.funcMap[func]
            if len(args) == 0:
                resp = funcobj()
            else:
                resp = funcobj(args)
            return resp
        except Exception as ex:
            return Exception('server exception, ' + str(ex))

    def setDiscoverConfig(self, config: DiscoveryConfig):
        self.discoveryConfig = config
        self.discovery = ConsulRpcDiscovery(self.discoveryConfig.consulHost, self.discoveryConfig.consulPort)

    def setKeepaliveTimeout(self, keepaliveTimeout: int):
        self.protocal.transport.setKeepaliveTimeout(keepaliveTimeout)


class SimpleTcpRpcServer(RpcServer):
    
    def __init__(self, host, port):
        RpcServer.__init__(self)
        self.host = host
        self.port = port
        self.protocal = TcpProtocal(host, port)
    
    def serve(self):
        self.protocal.transport.bind(self.keepaliveTimeout)
        while 1:
            msg = self.protocal.transport.recv()
            request = self.protocal.unserialize(msg)
            func, args = self.protocal.parseRequest(request)
            resp = self.run(func, args)
            self.protocal.transport.send(self.protocal.serialize(resp))


class TcpRpcServer(SimpleTcpRpcServer):

    def __init__(self, host, port, workers = multiprocessing.cpu_count()):
        SimpleTcpRpcServer.__init__(self, host, port)
        self.worker = workers

    def handle(self, conn):
        while 1:
            try:
                msg = self.protocal.transport.recvPeer(conn)
                request = self.protocal.unserialize(msg)
                func, args = self.protocal.parseRequest(request)
                resp = self.run(func, args)                    
                self.protocal.transport.sendPeer(self.protocal.serialize(resp), conn)
            except Exception as ex:
                conn.close()
                return

    def serve(self):
        self.protocal.transport.bind()
        if self.discovery and self.discoveryConfig:
            self.discovery.regist(self.discoveryConfig.serviceName, self.discoveryConfig.serviceHost, self.discoveryConfig.servicePort, ttlHeartBeat=True)
        while 1:
            conn, _ = self.protocal.transport.accept()
            t = threading.Thread(target=self.handle, args=(conn,) )
            t.start()


class HttpRpcServerController(SanicController):
    
    callBack = None

    async def handle(self):
        self.isRaw = True
        body = self.body()
        return self.callBack(body)

    @classmethod
    def setCallback(cls, callBack):
        cls.callBack = callBack


class HttpRpcServer(RpcServer, SanicController):
    
    def __init__(self, host, port, workers = multiprocessing.cpu_count()):
        RpcServer.__init__(self)
        self.host = host
        self.port = port
        self.worker = workers
        self.protocal = HttpProtocal(host, port, workers)
        HttpRpcServerController.setCallback(self.handle)
        self.protocal.transport.app.route('/', HttpRpcServerController)
        
    def handle(self, msg):
        request = self.protocal.unserialize(msg)
        func, args = self.protocal.parseRequest(request)
        resp = self.run(func, args)
        resp = self.protocal.serialize(resp)
        return resp

    def serve(self):
        if self.discovery and self.discoveryConfig:
            self.discovery.regist(self.discoveryConfig.serviceName, self.discoveryConfig.serviceHost, self.discoveryConfig.servicePort, ttlHeartBeat=True)
        self.protocal.transport.app.run()

    def disableLog(self):
        self.protocal.transport.app.disableLog()


"""
class AsyncTcpRpcServer(TcpRpcServer):

    def __init__(self, host, port):
        TcpRpcServer.__init__(self, host, port)
        self.host = host
        self.port = port
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.addr = (self.host, self.port)
        self.timeout = 10
        self.epoll = select.epoll()
        self.fdToSocket = {self.socket.fileno():self.socket}
        self.fdResp = {}

    def bind(self):
        self.socket.bind(self.addr)
        self.socket.listen(10)
        self.socket.setblocking(False)
        self.epoll.register(self.socket.fileno(), select.EPOLLIN)

    def serve(self):
        self.bind()
        while 1:
            events = self.epoll.poll(self.timeout)
            if not events:
                continue
            for fd, event in events:
                socket = self.fdToSocket[fd]
                if socket == self.socket:
                    conn, addr = self.socket.accept()
                    conn.setblocking(False)
                    self.epoll.register(conn.fileno(), select.EPOLLIN)
                    self.fdToSocket[conn.fileno()] = conn
                elif event & select.EPOLLHUP:
                    self.epoll.unregister(fd)
                    self.fdToSocket[fd].close()
                    del self.fdToSocket[fd]
                elif event & select.EPOLLIN:
                    msg = self.protocal.transport.recv(socket)
                    request = self.protocal.unserialize(msg)
                    func, args = self.protocal.parseRequest(request)
                    resp = self.run(func, args)
                    self.fdResp[fd] = resp
                    self.epoll.modify(fd, select.EPOLLOUT)
                elif event & select.EPOLLOUT:
                    if fd in self.fdResp:
                        resp = self.fdResp[fd]
                        self.protocal.transport.send(self.protocal.serialize(resp), socket)
                        self.epoll.modify(fd, select.EPOLLIN)
                        socket.close()
"""


class TornadoTcpRpcServer(TcpRpcServer):

    def __init__(self, host, port):
        TcpRpcServer.__init__(self, host, port)

    async def handleConnection(self, connection, address):
        stream = IOStream(connection)
        while 1:
            try:
                toread = 4
                readn = 0
                lengthbyte = b''
                while 1:
                    bufsize = toread - readn
                    if bufsize <= 0:
                        break
                    bytearr = await stream.read_bytes(bufsize, partial=True)
                    lengthbyte = lengthbyte + bytearr
                    readn = readn + len(bytearr)
                toread = struct.unpack("i", lengthbyte)[0]
                readn = 0
                msg = b''
                while 1:
                    bufsize = toread - readn
                    if bufsize <= 0:
                        break
                    bytearr = await stream.read_bytes(bufsize, partial=True)
                    msg = msg + bytearr
                    readn = readn + len(bytearr)
                request = self.protocal.unserialize(msg)
                func, args = self.protocal.parseRequest(request)
                resp = self.run(func, args)
                await stream.write( self.protocal.transport.getSendByte( self.protocal.serialize(resp) ) )
            except Exception as ex:
                connection.close()
                break

    def connectionReady(self, sock, fd, events):
        while True:
            try:
                connection, address = sock.accept()
                connection.setblocking(0)
                io_loop = tornado.ioloop.IOLoop.current()
                io_loop.spawn_callback(self.handleConnection, connection, address)
            except BlockingIOError:
                return
            
    def serve(self):
        self.protocal.transport.bind()
        self.protocal.transport.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.protocal.transport.socket.setblocking(0)
        self.ioLoop = tornado.ioloop.IOLoop.current()
        callback = functools.partial(self.connectionReady, self.protocal.transport.socket)
        self.ioLoop.add_handler(self.protocal.transport.socket.fileno(), callback, self.ioLoop.READ)
        self.ioLoop.start()


class UdpRpcServer(RpcServer):

    def __init__(self, host, port, workers = multiprocessing.cpu_count()):
        RpcServer.__init__(self)
        self.protocal = UdpProtocal(host, port)
        self.worker = workers
        self.queueMaxSize = 100000
        self.queue = queue.Queue(self.queueMaxSize)

    def startWorkers(self):
        for i in range(self.worker):
            t = threading.Thread(target=self.handle)
            t.start()

    def handle(self):
        while 1:
            try:
                body = self.queue.get()
                addr = body.get('addr')
                msg = body.get('msg')
                request = self.protocal.unserialize(msg)
                func, args = self.protocal.parseRequest(request)
                resp = self.run(func, args)
                self.protocal.transport.sendPeer(self.protocal.serialize(resp), addr = addr)
            except Exception as ex:
                print('udp handler exception:', ex)
    
    def serve(self):
        self.startWorkers()
        self.protocal.transport.bind()
        if self.discovery and self.discoveryConfig:
            self.discovery.regist(self.discoveryConfig.serviceName, self.discoveryConfig.serviceHost, self.discoveryConfig.servicePort, ttlHeartBeat=True)
        while 1:
            try:
                msg, cliAddr = self.protocal.transport.recv()
                self.queue.put({'msg' : msg, 'addr' : cliAddr})
            except socket.timeout:
                pass


#codiing=utf-8
'''
pip install fabric
'''

from fabric.api import *
from fabric.context_managers import *
import sys


def batchSetHosts(serverList, hostList, delHostList=[]):
    """
	This function will set /etc/hosts.If the host
	has been exist in hosts file, then it will be
	overwrite, else it will be append to the hosts
	file.

	:param serverList list: a list of {'host':'xxx', 'port':'xx', 'user':'xx', 'password':''xx}
	:param hostList list: a list of {'ip domain1 domain2'}
	:param delHostList list: a list of domain
	"""
    def sortedByKeys(dic):
        keys = dic.keys()
        keys.sort()
        retDic = {}
        for key in keys:
            retDic[key] = dic[key]
        return retDic

    for server in serverList:
        env.host_string = server['host']
        env.port = server['port']
        env.user = server['user']
        env.password = server['password']
        ret = sudo('cat /etc/hosts')
        lines = ret.split('\n')
        hostHash = {}
        newHostList = []
        for line in lines:
            line = line.replace('\n', '')
            line = line.replace('\r', '')
            line = line.strip()
            if line == '':
                continue
            if line.replace(' ', '') == '':
                continue
            if line[0] == '#':
                newHostList.append(line)
                continue
            items = line.split(' ')
            ip = items[0].strip()
            for i in xrange(len(items)):
                if i == 0:
                    continue
                domain = items[i]
                domain = domain.strip()
                if domain == '':
                    continue
                if domain in delHostList:
                    continue
                hostHash[domain] = ip
        setHostHash = {}
        for host in hostList:
            host = host.strip()
            items = host.split(' ')
            ip = items[0].strip()
            for i in xrange(len(items)):
                if i == 0:
                    continue
                domain = items[i]
                domain = domain.strip()
                if domain == '':
                    continue
                if domain in delHostList:
                    continue
                setHostHash[domain] = ip
        for domain, ip in setHostHash.items():
            hostHash[domain] = ip
        hostHash = sortedByKeys(hostHash)
        for domain, ip in hostHash.items():
            hostline = ip + ' ' + domain
            newHostList.append(hostline)
        newHostList.sort()
        hosts = '\n'.join(newHostList)
        sudo("echo '%s' > /etc/hosts" % (hosts))


def exeCmd(cmd):
    return sudo(cmd)


def batchLoopExec(serverList, cmdList):
    """
	This function will exec cmd on the server in
	hostList one by one. The commond will be execute on
	next server when the last server has ran the command.
	"""

    for server in serverList:
        #env.hosts = [ server['host'] ]
        env.host_string = server['host']
        env.port = server['port']
        env.user = server['user']
        env.password = server['password']
        for cmd in cmdList:
            exeCmd(cmd)


def batchSyncExec(serverList, cmdList):
    """
	This function will exec cmd on the server with cmd
	step by step instead of by server on by on.So the same
	command will execute on all server, then the next command
	will be execute.
	"""
    for cmd in cmdList:
        for server in serverList:
            env.host_string = server['host']
            env.port = server['port']
            env.user = server['user']
            env.password = server['password']
            exeCmd(cmd)



#coding=utf-8

def page_by_result(res_list, pagenum, perpage):
    ret = []
    total_count = len(res_list)
    if total_count == 0:
        return []
    if pagenum == 0:
        pagenum = 1
    if perpage < 0:
        perpage = 30
    begin = (pagenum - 1 ) * perpage
    end = begin + perpage
    if begin > total_count:
        return []
    elif end < total_count:
        return res_list[begin:end]
    elif end >= total_count:
        return res_list[begin:]
    else:
        return []


class PageLimit(object):
    
    def __init__(self, totalCount, perPage, curPage):
        self.totalCount = totalCount
        self.perPage = perPage
        self.curPage = curPage

    def page(self):
        begin = self.curPage * self.perPage - self.perPage
        end = begin + self.perPage
        if begin > self.totalCount:
            begin = 0
            end = self.perPage
        return begin, end


#coding=utf-8
'''
小表同步
sync_tables(
    '192.168.1.1',
    3306,
    'xxxx',
    'xxxx',
    'db1',

    '192.168.1.2',
    3306,
    'xxxx',
    'xxxx',
    'db1',

    ['users']
)
'''

import sys
reload(sys)
sys.setdefaultencoding('utf-8')
from db import DB
import time

SYNC_FAILED_TABLES = []


def get_last_sync_failed_tables():
    return SYNC_FAILED_TABLES


def sync_one_table(src_db, dst_db, table, limit=1000):
    cnt = int(src_db.query('select count(1) from %s' % table)[0]['count(1)'])
    if cnt > limit:
        return False
    create_table_tag = False
    sql = 'show tables'
    rows = dst_db.query(sql)
    if rows == None or len(rows) == 0:
        create_table_tag = True
    else:
        tables = [row.values()[0] for row in rows]
        if table not in tables:
            create_table_tag = True
    if create_table_tag:
        sql = 'show create table %s' % table
        rows = src_db.query(sql)
        if rows == None or len(rows) == 0:
            pass
        else:
            row = rows[0]
            sql = row['Create Table']
            dst_db.query(sql)
    sql = 'select * from %s' % table
    rows = src_db.query(sql)
    if rows == None or len(rows) == 0: return False
    row = rows[0]
    columns = row.keys()
    columns = ','.join(columns)
    sql = "replace into %s(%s) values" % (table, columns)
    for row in rows:
        sql = sql + "('" + "','".join([str(val)
                                       for val in row.values()]) + "'),"
    sql = sql[:-1]
    dst_db.update(sql)
    return True


def sync_tables(src_db_host,
                src_db_port,
                src_db_user,
                src_db_pwd,
                src_db_name,
                dst_db_host,
                dst_db_port,
                dst_db_user,
                dst_db_pwd,
                dst_db_name,
                tables=[],
                sleep_intval=0.01,
                limit=1000):
    '''
    小表的同步
    '''
    if tables == None:
        return False

    global SYNC_FAILED_TABLES
    SYNC_FAILED_TABLES = []

    src_db = DB(src_db_host, src_db_port, src_db_user, src_db_pwd, src_db_name)

    dst_db = DB(dst_db_host, dst_db_port, dst_db_user, dst_db_pwd, dst_db_name)

    all_sync_succed_tag = True

    for table in tables:
        time.sleep(sleep_intval)
        tag = sync_one_table(src_db, dst_db, table, limit)
        if tag == False:
            all_sync_succed_tag = False
            SYNC_FAILED_TABLES.append(table)

    return all_sync_succed_tag



#coding=utf-8
'''
这里调用jira api创建issue
参考:
https://developer.atlassian.com/server/jira/platform/jira-rest-api-examples/
https://developer.atlassian.com/cloud/jira/platform/rest/#api-api-2-issue-issueIdOrKey-transitions-post

curl -D- -u 用户名:密码 -X POST --data '{"fields": {"project":{ "key": "OP"},"summary": "jira api test","description": "create issue test","components":[{"name":"MySQL"}], "issuetype": {"name":"任务"}}}' -H "Content-Type: application/json" http://xxxxx/rest/api/2/issue/
'''

import requests
import ujson


def create_issue(url,
                 username,
                 password,
                 project_key='OP',
                 issuetype=u'任务',
                 components='MySQL',
                 summary='',
                 description=''):
    '''
    url: http://xxxxxx/rest/api/2/issue/
    '''
    headers = {'Content-Type': 'application/json'}
    params = {
        'fields': {
            'project': {
                'key': project_key
            },
            'summary': summary,
            'components': [{
                'name': components
            }],
            'description': description,
            'issuetype': {
                "name": issuetype
            }
        }
    }
    params = ujson.dumps(params)
    s = requests.Session()
    s.headers.update(headers)
    s.auth = (username, password)
    r = s.post(url, data=params)
    return r.status_code, r.text


def reslove_issue(url, username, password, issue_key, trans_id):
    '''
    url: https://xxxx/rest/api/2/issue/issueIdOrKey/transitions
    '''
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json'
    }
    params = {'transition': {'id': trans_id}}
    params = ujson.dumps(params)
    s = requests.Session()
    s.headers.update(headers)
    s.auth = (username, password)
    url = url.replace('issueIdOrKey', issue_key)
    r = s.post(url, data=params)
    return r.status_code, r.text



#coding=utf-8

from multiprocessing import Process
from kazoo.client import KazooClient
import platform
import time


def log_info(logger, string):
    if logger: logger.info('[zk] ' + string)


def log_warn(logger, string):
    if logger: logger.warning('[zk] ' + string)


def log_error(logger, string):
    if logger: logger.error('[zk] ' + string)


zk = None


def register(zknodes, path, child_node_name, child_node_value, logger):
    if platform.python_version()[0] != '2':
        child_node_value = child_node_value.encode('utf-8')
    global zk
    if zk == None:
        zk = KazooClient(hosts=zknodes)
        zk.start()
    if not zk.exists(path):
        log_warn(logger, "%s not exist" % path)
        zk.ensure_path(path)
        log_info(logger, "create %s" % path)
    if path[-1] != '/': path = path + '/'
    k = path + child_node_name
    if not zk.exists(k):
        log_warn(logger, "%s not exist" % k)
        zk.create(k, child_node_value, ephemeral=True)
        log_info(logger, "create %s" % k)
    else:
        zk.set(k, child_node_value)
    children = zk.get_children(path)
    log_info(logger, "get current childrent:" + str(children))


def loop_register(zknodes,
                  path,
                  child_node_name,
                  child_node_value,
                  intval=30,
                  logger=None,
                  is_regist_callback=None):
    while True:
        try:
            is_regist = True
            if is_regist_callback:
                is_regist = is_regist_callback(child_node_name,
                                               child_node_value)
            if is_regist:
                register(zknodes, path, child_node_name, child_node_value,
                         logger)
                log_info(logger, "regist succed")
            else:
                global zk
                if zk:
                    zk.stop()
                    zk.close()
                    zk = None
                log_error(logger,
                          "is_regist_callback check failed, not regist")
        except Exception as ex:
            log_error(logger, "regist failed:" + str(ex))
        time.sleep(intval)


def start_loop_register(zknodes,
                        path,
                        child_node_name='',
                        child_node_value='',
                        intval=30,
                        logger=None,
                        is_regist_callback=None):
    p = Process(target=loop_register,
                args=(zknodes, path, child_node_name, child_node_value, intval,
                      logger, is_regist_callback))
    p.start()
    return


def get_children(zknodes, path):
    zkcli = KazooClient(hosts=zknodes)
    zkcli.start()
    if path[-1] != '/': path = path + '/'
    children = zkcli.get_children(path)
    m = {}
    for child in children:
        m[child] = zkcli.get(path + child)[0]
    zkcli.stop()
    zkcli.close()
    return m


'''
def is_regist(node_name, node_val):
    return True

num = 1
def test(node_name, node_val):
    global num
    num = num + 1
    if num > 30:
        return True
    if num > 10:
        return False
    return True

from log import Log
log = Log("./debug.log")
log.setOutput(True)
start_loop_register('127.0.0.1:2181', '/test/1', 'a', 'a', 3, log, is_regist_callback=is_regist)
start_loop_register('127.0.0.1:2181', '/test/1', 'b', 'b', 3, log, is_regist_callback=test)
'''



#coding=utf-8

from agileutil.consul import ConsulApi, Instance
import time
import threading


class RpcDiscovery(object):
    pass


class DiscoveryConfig(object):

    def __init__(self, consulHost, consulPort, serviceName, serviceHost = '', servicePort = 0):
        self.consulHost = consulHost
        self.consulPort = consulPort
        self.serviceName = serviceName
        self.serviceHost = serviceHost
        self.servicePort = servicePort


class ConsulRpcDiscovery(RpcDiscovery):

    def __init__(self, consulHost, consulPort):
        self.consulHost = consulHost
        self.consulPort = consulPort
        self.consulApi = ConsulApi(consulHost, consulPort)
        self.instanceList = []
        self.heartbeatInterval = 10

    def doHeartbeat(self, service, address, port, interval):
        while 1:
            try:
                self.consulApi.ttlHeartbeat(service, address, port)
            except:
                pass
            time.sleep(interval)

    def regist(self, service, address, port, ttlHeartBeat = True):
        self.consulApi.registService(serviceName = service, address = address, port=port)
        if ttlHeartBeat:
            t = threading.Thread(target=self.doHeartbeat, args=(service, address, port, self.heartbeatInterval))
            t.start()

    def getInstanceList(self, service):
        instanceList = self.consulApi.getServiceInstanceList(service)
        return instanceList


#coding=utf-8
'''
usage:

MemQueue().getInstance().push('1').push('2')
print( MemQueue().getInstance().pop() )
'''

import time


class MemQueue:
    def __init__(self, asyncTag=True):
        self.queue = []
        self.asyncTag = asyncTag

    instance = None

    @classmethod
    def getInstance(cls):
        if cls.instance == None:
            cls.instance = MemQueue()
        return cls.instance

    def push(self, elem):
        self.queue.append(elem)
        return self

    def pop(self):
        elem = None

        if self.asyncTag:
            try:
                elem = self.queue.pop()
            except:
                pass
            return elem

        while 1:
            try:
                elem = self.queue.pop()
            except:
                time.sleep(0.01)
                continue
            return elem

    def count(self):
        return len(self.queue)


class UniMemQueue(MemQueue):
    def __init__(self, asyncTag=True):
        super().__init__(asyncTag)
        self.queue = set()

    @classmethod
    def getInstance(cls):
        if cls.instance == None:
            cls.instance = UniMemQueue()
        return cls.instance

    def push(self, elem):
        self.queue.add(elem)
        return self



#coding=utf-8
'''
该模块目前只支持Python2.7

例子：

#coding=utf-8
from agileutil.remote_manage import Server, ServerList

serverList = ServerList (
	Server(ip='192.168.1.1', sshPort=22, sshUser='xxx', sshPwd='xxx'),
	Server(ip='192.168.1.2', sshPort=22, sshUser='xxx', sshPwd='xxx')
)

cmdList = [
	'df -lh',
	'ps -ef'
]

serverList.execCmdList(cmdList)

'''

import paramiko


class Server:
    def __init__(self, ip='', sshPort=22, sshUser='', sshPwd='', timeout=10):
        self.ip = ip
        self.sshPort = sshPort
        self.sshUser = sshUser
        self.sshPwd = sshPwd
        self.timeout = timeout

    def execCmdList(self, cmdList=[]):
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        print('-----------------------------------')
        try:
            client.connect(hostname=self.ip,
                           port=self.sshPort,
                           username=self.sshUser,
                           password=self.sshPwd,
                           timeout=self.timeout)
            print('connect %s:%s succed' % (self.ip, self.sshPort))
        except Exception as ex:
            print('connect %s:%s failed:%s' % (self.ip, self.sshPort, str(ex)))
            return
        
        ssh = client.invoke_shell()
        if self.sshUser != 'root':
            ssh.send('sudo -s \n')
            buff = ''
            while not buff.endswith('[sudo] password for %s: ' % self.sshUser):
                resp = ssh.recv(9999)
                buff = buff + resp.decode('utf-8')
            ssh.send(self.sshPwd)
            ssh.send('\n')
            buff = ''
            while not buff.endswith('# '):
                resp = ssh.recv(9999)
                buff = buff + resp

            currentUser = 'root'
        else:
            currentUser = self.sshUser

        for cmd in cmdList:
            print('cmd:', cmd)
            ssh.send(cmd + '\n')
            buff = ''
            while not buff.endswith("\r\n"):
                resp = ssh.recv(1)
                buff = buff + resp
            buff = ''
            while not buff.endswith('# '):
                resp = ssh.recv(1)
                buff = buff + resp
            output = buff.split('%s@' % currentUser)[0]
            if output.endswith('\r\n'):
                output = output.strip('\r\n')
            print('output:', output)

        client.close()


class ServerList:
    def __init__(self):
        self.sList = []

    def append(self, serverItem):
        self.sList.append(serverItem)

    def execCmdList(self, cmdList=[]):
        for s in self.sList:
            s.execCmdList(cmdList)



#coding=utf-8

import re
try:
    from functools import reduce
except:
    pass
import socket
import struct


def get_mask_by_ip_range(ipBegin, ipEnd):
    """
    get ip range mask
    :param ipBegin:
    :param ipEnd:
    :return:
    """
    ipBeginList = ipBegin.strip().split('.')
    ipEndList = ipEnd.strip().split('.')
    for i in range(4):
        ipRange = str(ipBeginList[i])
        fullRange = ''
        length = len(ipRange)
        if length < 3:
            rest = 3 - length
            for j in xrange(rest):
                fullRange = fullRange + '0'
        fullRange = fullRange + ipRange
        ipBeginList[i] = fullRange
    for i in xrange(4):
        ipRange = str(ipEndList[i])
        fullRange = ''
        length = len(ipRange)
        if length < 3:
            rest = 3 - length
            for j in xrange(rest):
                fullRange = fullRange + '0'
        fullRange = fullRange + ipRange
        ipEndList[i] = fullRange

    ipBinBegin = ''
    for i in ipBeginList:
        ipBin = str(bin(int(i))).replace('0b', '')
        length = len(ipBin)
        if length < 8:
            rest = 8 - length
            for j in xrange(rest):
                ipBin = '0' + ipBin
        ipBinBegin = ipBinBegin + ipBin
    ipBinEnd = ''
    for i in ipEndList:
        ipBin = str(bin(int(i))).replace('0b', '')
        length = len(ipBin)
        if length < 8:
            rest = 8 - length
            for j in xrange(rest):
                ipBin = '0' + ipBin
        ipBinEnd = ipBinEnd + ipBin
    if len(ipBinBegin) != len(ipBinEnd):
        return False
    sameTimes = 0
    for i in xrange(32):
        if ipBinBegin[i] == ipBinEnd[i]:
            sameTimes = sameTimes + 1
        else:
            break
    restTimes = 32 - sameTimes
    maskBinStr = ''
    for i in xrange(sameTimes):
        maskBinStr = maskBinStr + '1'
    for i in xrange(restTimes):
        maskBinStr = maskBinStr + '0'
    range1 = ''
    range2 = ''
    range3 = ''
    range4 = ''
    for i in [0, 1, 2, 3, 4, 5, 6, 7]:
        range1 = range1 + maskBinStr[i]
    for i in [8, 9, 10, 11, 12, 13, 14, 15]:
        range2 = range2 + maskBinStr[i]
    for i in [16, 17, 18, 19, 20, 21, 22, 23]:
        range3 = range3 + maskBinStr[i]
    for i in [24, 25, 26, 27, 28, 29, 30, 31]:
        range4 = range4 + maskBinStr[i]
    intRange1 = int(range1, 2)
    intRange2 = int(range2, 2)
    intRange3 = int(range3, 2)
    intRange4 = int(range4, 2)
    maskStr = "%s.%s.%s.%s" % (intRange1, intRange2, intRange3, intRange4)
    return maskStr


def get_host_no(ipBegin, ipEnd):
    ipBeginList = ipBegin.strip().split('.')
    ipEndList = ipEnd.strip().split('.')
    for i in range(4):
        ipRange = str(ipBeginList[i])
        fullRange = ''
        length = len(ipRange)
        if length < 3:
            rest = 3 - length
            for j in xrange(rest):
                fullRange = fullRange + '0'
        fullRange = fullRange + ipRange
        ipBeginList[i] = fullRange
    for i in xrange(4):
        ipRange = str(ipEndList[i])
        fullRange = ''
        length = len(ipRange)
        if length < 3:
            rest = 3 - length
            for j in xrange(rest):
                fullRange = fullRange + '0'
        fullRange = fullRange + ipRange
        ipEndList[i] = fullRange

    ipBinBegin = ''
    for i in ipBeginList:
        ipBin = str(bin(int(i))).replace('0b', '')
        length = len(ipBin)
        if length < 8:
            rest = 8 - length
            for j in xrange(rest):
                ipBin = '0' + ipBin
        ipBinBegin = ipBinBegin + ipBin
    ipBinEnd = ''
    for i in ipEndList:
        ipBin = str(bin(int(i))).replace('0b', '')
        length = len(ipBin)
        if length < 8:
            rest = 8 - length
            for j in xrange(rest):
                ipBin = '0' + ipBin
        ipBinEnd = ipBinEnd + ipBin
    if len(ipBinBegin) != len(ipBinEnd):
        return False
    sameTimes = 0
    for i in xrange(32):
        if ipBinBegin[i] == ipBinEnd[i]:
            sameTimes = sameTimes + 1
        else:
            break
    return sameTimes


def ip_range_to_cidr(ipBegin, ipEnd):
    ipBeginList = ipBegin.strip().split('.')
    ipEndList = ipEnd.strip().split('.')
    ipBeginStr = ''.join(ipBeginList)
    ipEndStr = ''.join(ipEndList)
    if len(ipBeginStr) != len(ipEndStr):
        return False
    length = len(ipBeginStr)
    sameTimes = 0
    for i in xrange(length):
        if ipBeginStr[i] == ipEndStr[i]:
            sameTimes = sameTimes + 1
        else:
            break


def remove_zero(ip):
    ipList = ip.strip().split('.')
    for i in xrange(4):
        partStr = str(int(ipList[i]))
        ipList[i] = partStr
    ip = '.'.join(ipList)
    return ip


def compress_cidr_list(cidrList):
    """
    return cidr list after compressed and ipCount
    """
    cidrHash = {}
    compressedCidrList = []
    ipCount = 0
    for cidr in cidrList:
        ip, prefix = cidr.strip().split('/')
        if not cidrHash.has_key(prefix):
            cidrHash[prefix] = []
        cidrHash[prefix].append(cidr)
    from IPy import IP, IPSet
    for prefix, cidrList in cidrHash.items():
        ipSetList = []
        for cidr in cidrList:
            ipSetList.append(IP(cidr, make_net=1))
        ipSet = IPSet(ipSetList)
        for compressdCidr in ipSet:
            compressedCidrList.append(str(compressdCidr))
            ipCount = ipCount + compressdCidr.len()
    return compressedCidrList, ipCount


def ip_to_int(ip):
    return reduce(lambda x, y: (x << 8) + y, map(int, ip.split('.')))


def is_internal_ip(ip):
    ip = ip_to_int(ip)
    net_a = ip_to_int('10.255.255.255') >> 24
    net_b = ip_to_int('172.31.255.255') >> 20
    net_c = ip_to_int('192.168.255.255') >> 16
    return ip >> 24 == net_a or ip >> 20 == net_b or ip >> 16 == net_c


def is_invalid_ip(ip):
    p = re.compile(
        '^((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$')
    if p.match(ip): return True
    else: return False


def ip_to_int(ip):
    intVal = socket.ntohl(struct.unpack("I", socket.inet_aton(str(ip)))[0])
    return intVal



#!/usr/bin/env python
#coding=utf-8
'''
pip install pymysql
pip install web.py
'''

import web
import pymysql
import json

DB_HOST = '127.0.0.1'
DB_PORT = 3306
DB_USER = 'xxx'
DB_PWD = 'xxxx'
DB_NAME = 'xxx'
DB_CHARSET = 'xxxx'

urls = ('/intf/get_servers_by_rack', 'get_servers_by_rack')


def getConnection():
    conn = pymysql.connect(host=DB_HOST,
                           port=DB_PORT,
                           user=DB_USER,
                           password=DB_PWD,
                           db=DB_NAME,
                           charset=DB_CHARSET,
                           cursorclass=pymysql.cursors.DictCursor)
    return conn


def query(sql):
    conn = getConnection()
    cursor = conn.cursor()
    cursor.execute(sql)
    rows = cursor.fetchall()
    cursor.close()
    conn.close()
    return rows


def update(sql):
    conn = getConnection()
    cursor = conn.cursor()
    effect_row = cursor.execute(sql)
    conn.commit()
    cursor.close()
    conn.close()
    return effect_row


def auth(func):
    def wrapper(*args, **kw):
        i = web.input()
        app = i.get('app')
        if app == None or app not in ['test']:
            return web.Forbidden()
        return func(*args, **kw)

    return wrapper


def get(k):
    i = web.input()
    v = ''
    try:
        v = i[k]
    except:
        pass
    return v


class get_servers_by_rack:
    def GET(self):
        return self.REQUEST()

    def POST(self):
        return self.REQUEST()

    def REQUEST(self):
        rackName = get('rack_name')
        if rackName == '': return ''
        sql = "select id from rack_infos where name = '%s'" % (rackName)
        rows = query(sql)
        if rows == None or len(rows) == 0: return ''
        row = rows[0]
        rackId = row['id']
        sql = "select * from servers where rack_id=%s" % (rackId)
        rows = query(sql)
        if rows == None or len(rows) == 0: return ''
        for r in rows:
            r['pandian_time'] = str(r['pandian_time'])
            r['updated_at'] = str(r['updated_at'])
            r['created_at'] = str(r['created_at'])
        return json.dumps(rows)


if __name__ == '__main__': web.application(urls, globals()).run()
'''
sys.argv.append(config('PORT'))
urls = router.routers
app = web.application(urls, globals())
web.config.session_parameters['timeout'] = config('SESSION_EXPIRE', cast=int) 
session = web.session.Session(app, web.session.DiskStore(config('SESSION_PATH')))

def session_hook():
    web.ctx.session = session
    
app.add_processor(web.loadhook(session_hook))
application = app.wsgifunc()

if __name__ == '__main__': 
    app.run()
'''



#coding=utf-8

'''
python 2.7+
pip install selenium
download chromedriver from https://npm.taobao.org/mirrors/chromedriver/
'''

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
import os
import traceback
import time

class Screenshot(object):

    def __init__(self, url, width, height, driverPath, pic, sleepIntval = 10):
        self.url = url
        self.width = width
        self.height = height
        self.driverPath = driverPath
        self.pic = pic
        self.sleepIntval = sleepIntval

    def save(self):
        '''
        保存截图
        成功返回None, 否则返回错误信息
        '''
        try:
            os.environ["webdriver.chrome.driver"] = self.driverPath
            chrome_options = Options()
            chrome_options.add_argument('--no-sandbox')
            chrome_options.add_argument('--disable-dev-shm-usage')
            chrome_options.add_argument('--headless')
            driver = webdriver.Chrome(self.driverPath, chrome_options=chrome_options)
            driver.set_window_size(self.width, self.height)
            driver.get(self.url)
            time.sleep(self.sleepIntval)
            driver.save_screenshot(self.pic)
            driver.close()
            return None
        except Exception as ex:
            return str(ex) + traceback.format_exc()


#coding=utf-8

import ujson
import web
from web.contrib.template import render_jinja
import sys
try:
    from imp import reload
except:
    pass
reload(sys)
try:
    sys.setdefaultencoding('utf-8')
except:
    pass
import platform
from sanic.response import json, text


class SanicBase:
    def GET(self):
        return text(self.handle())

    def POST(self):
        return text(self.handle())

    def handle(self):
        return ''

    def req(self, k):
        pass



#coding=utf-8

import os
import re
import sys
import socket
import hashlib
import urllib
try:
    import urllib2
except:
    import urllib as urllib2
import base64
import time
try:
    import commands
except:
    import subprocess as commands
#import sqlparse
import zlib
import platform


def md5(str):
    m = hashlib.md5()
    m.update(str)
    return m.hexdigest()


def itoa(ascii_value):
    return chr(ascii_value)


def atoi(char):
    return ord(char)


def exit(text=None):
    if text:
        print(text)
    sys.exit()


def findall(regex, text):
    pattern = re.compile(regex)
    search_result = pattern.findall(text)
    return search_result


def create_standard_daemon():
    if os.fork() > 0:
        sys.exit(0)
    os.chdir('/')
    os.setsid()
    os.umask(0)
    if (os.fork() > 0):
        sys.exit(0)
    return True


def create_simple_daemon():
    if (os.fork() > 0):
        sys.exit(0)
    return True


def execmd(cmdString):
    status, output = commands.getstatusoutput(cmdString)
    return status, output


def http(url, params=None, mtimeout=10, user=None, pwd=None):
    if user and pwd:
        #auth
        base64String = base64.encodestring("%s:%s" % (user, pwd))
        authheader = "Basic %s" % base64String
        request = urllib2.Request(url)
        request.add_header("Authorization", authheader)
        response = urllib2.urlopen(request)
        return response.code, response.read()
    else:
        code = 0
        try:
            data = None
            if params:
                data = urllib.urlencode(params)
            request = urllib2.Request(url, data)
            response = urllib2.urlopen(url=request, timeout=mtimeout)
            return response.code, response.read()
        except Exception as ex:
            return code, str(ex)


def safe_reload(module_name):
    try:
        reload(module_name)
    except Exception as ex:
        pass


def write_file(file, content, is_append=False):
    way = 'w+'
    if is_append:
        way = 'a+'
    try:
        fd = open(file, way)
        fd.write(content)
        fd.close()
        return True
    except:
        return False


def get_file_content(file):
    fd = open(file, 'r')
    file_content = fd.read()
    fd.close()
    return file_content


def download(url, filename):
    """
    download file and save to spefic path 
    """
    try:
        urllib.urlretrieve(url, filename)
    except Exception as ex:
        return False
    return True


def hostname():
    return socket.gethostname()


def local_ip():
    return socket.gethostbyname(socket.gethostname())


def local_ipv4_list():
    addrs = socket.getaddrinfo(socket.gethostname(), None)
    if addrs == None or len(addrs) == 0: return []
    ipv4list = []
    for addr in addrs:
        tuple_element = addr[4]
        ip = tuple_element[0]
        if ':' in ip: continue
        ipv4list.append(ip)
    ipv4list = list(set(ipv4list))
    return ipv4list


def telnet(ip, port, timeout):
    socket.setdefaulttimeout(timeout)
    client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    client.connect((ip, port))
    client.close()
    return True


def proc_num_by_key_word(kwd):
    status, output = execmd("ps auxww | grep %s | grep -v grep | wc -l" % kwd)
    return int(output)


def del_file_before_days(dirname, days):
    if dirname[-1] != '/':
        dirname = dirname + '/'
    f_list = os.listdir(dirname)
    cur_time = time.time()
    for f in f_list:
        abs_path = dirname + f
        modify_time = os.path.getmtime(abs_path)
        time_diff = cur_time - modify_time
        days_diff = (time_diff / 3600) / 24
        if days_diff >= days:
            os.remove(abs_path)


def get_cur_proc_socket_num():
    pid = os.getpid()
    cmd = "ls /proc/%s/fd -l | grep socket: | wc -l" % pid
    _, output = execmd(cmd)
    return int(output.strip())


def file_size(f):
    return os.path.getsize(f)


def add_log_not_roll_event(logfile,
                           cmd_list=[],
                           is_block=False,
                           check_intval=5,
                           exec_intval=1,
                           is_raise_open_file_exception=True):
    ########################################
    def loop_check(logfile,
                   cmd_list=[],
                   check_intval=5,
                   exec_intval=1,
                   is_raise_open_file_exception=True):
        last_size = 0
        while True:
            time.sleep(check_intval)
            size = 0
            try:
                size = file_size(logfile)
            except Exception as ex:
                if is_raise_open_file_exception:
                    raise ex
                else:
                    size = 0
            print('size', size)
            if size == last_size:
                for cmd in cmd_list:
                    time.sleep(exec_intval)
                    os.system(cmd)
            else:
                last_size = size

    ########################################
    if is_block:
        loop_check(logfile, cmd_list, check_intval, exec_intval)
    else:
        pid = os.fork()
        if pid > 0:
            return True
        elif pid < 0:
            return False
        else:
            loop_check(logfile, cmd_list, check_intval, exec_intval)


#def format_sql(sql):
#    if '\n' in sql: return sql
#    formatSqlString = sqlparse.format(sql, reindent=True, keyword_case='upper')
#    return formatSqlString


def compress(string, level=9, encoding='utf-8'):
    if platform.python_version()[0:1] == '3':
        string = string.encode(encoding)
        compress = zlib.compress(string, level)
        return compress
    else:
        return zlib.compress(string)


def decompress(string):
    de_com = zlib.decompress(string)
    return de_com


def list_all_files(dir_path):
    files = []
    l = os.listdir(dir_path)
    for i in range(0, len(l)):
        path = os.path.join(dir_path, l[i])
        if os.path.isdir(path):
            files.extend(list_all_files(path))
        if os.path.isfile(path):
            files.append(path)
    return files


def gen_reqs():
    py_version = ''
    if platform.python_version()[0:1] == '3':
        py_version = 3
    else:
        py_version = ''
    pip = 'pip' + str(py_version)
    cmd = 'source ./bin/activate'
    execmd(cmd)
    cmd = pip + " freeze 2> /dev/null"
    status, output = execmd(cmd)
    lines = output.split("\n")
    lines = list(set(lines))
    try:
        lines.remove('')
    except:
        pass
    pkg_ver_map = {}
    for line in lines:
        pkg, ver = line.split('==')
        pkg_ver_map[pkg] = ver
    include_pkg = []
    all_f = list_all_files('./')
    all_py_f = []
    for f in all_f:
        _, ext = os.path.splitext(f)
        if ext == '.py':
            all_py_f.append(f)
    all_import_lines = []
    for f in all_py_f:
        fp = open(f, 'r')
        lines = fp.readlines()
        fp.close()
        for line in lines:
            if 'import ' in line or 'from ' in line:
                all_import_lines.append(line)
    real_need_imports_map = {}
    for pkg, ver in pkg_ver_map.items():
        tmp_pkg = pkg.replace('python', '').replace('-', '')
        for line in all_import_lines:
            if tmp_pkg in line:
                real_need_imports_map[pkg] = ver
                break
    f = open('./requirements.txt', 'w')
    for pkg, ver in real_need_imports_map.items():
        f.write(pkg + '==' + str(ver) + "\n")
    f.close()


class DebugUtil(object):

    def __init__(self):
        self.tagList = []
        self.curTime = time.time()

    def tag(self, tagName):
        if len(self.tagList) == 0:
            self.tagList.append(tagName)
            self.curTime = time.time()
        else:
            diff = time.time() - self.curTime
            self.curTime = time.time()
            self.tagList.append(round(diff, 3))
            self.tagList.append(tagName)

    def finish(self):
        cost = 0
        for i in self.tagList:
            print(i)
            if type(i) == int or type(i) == float:
                cost = cost + i
        print('total cost:' + str(cost) + ' seconds')


#coding=utf-8
'''
pip install pymysql
一些旧系统使用，为了兼容保留此文件
'''

import pymysql
from .threadlock import ThreadLock


class DB(object):

    __slots__ = [
        '__host', '__port', '__user', '__passwd', '__dbName', '__conn',
        '__cur', '__ispersist', '__log', '__is_mutex', '__connect_timeout',
        '__read_timeout'
    ]

    def __init__(self,
                 host,
                 port,
                 user,
                 passwd,
                 dbName,
                 ispersist=False,
                 log=None,
                 is_mutex=False,
                 connect_timeout=10,
                 read_timeout=None):
        self.__host = host
        self.__port = port
        self.__user = user
        self.__passwd = passwd
        self.__dbName = dbName
        self.__conn = None
        self.__cur = None
        self.__ispersist = ispersist
        self.__log = log
        self.__is_mutex = is_mutex
        self.__connect_timeout = connect_timeout
        self.__read_timeout = read_timeout

        if (self.__ispersist):
            DB.connect(self)

    def log_error(self, err_info):
        if (self.__log):
            self.__log.error(err_info)

    def connect(self):
        try:
            self.__conn = pymysql.Connect(
                host=self.__host,
                port=self.__port,
                user=self.__user,
                passwd=self.__passwd,
                db=self.__dbName,
                charset='utf8',
                cursorclass=pymysql.cursors.DictCursor,
                connect_timeout=self.__connect_timeout,
                read_timeout=self.__read_timeout)
            self.__cur = self.__conn.cursor()
        except Exception as ex:
            self.log_error('db connect exception: ' + str(ex))
            raise ex

    def close(self):
        try:
            self.__cur.close()
            self.__conn.close()
        except Exception as ex:
            self.log_error('db close exception: ' + str(ex))

    def lastrowid(self):
        return self.__cur.lastrowid

    def lock(self):
        if self.__is_mutex:
            ThreadLock.lock()

    def unlock(self):
        if self.__is_mutex:
            ThreadLock.unlock()

    def update(self, sql):
        self.lock()

        if (not self.__ispersist):
            try:
                self.connect()
            except Exception as ex:
                self.unlock()
                raise ex
        else:
            try:
                self.__conn.ping(True)
            except Exception as ex:
                self.log_error(str(ex))
                try:
                    self.connect()
                except Exception as ex:
                    self.unlock()
                    raise ex

        effect_rows = 0
        try:
            effect_rows = self.__cur.execute(sql)
            self.__conn.commit()
        except Exception as ex:
            self.unlock()
            self.log_error('db update exception: ' + str(ex))
            raise ex

        if (not self.__ispersist):
            try:
                self.close()
            except Exception as ex:
                self.unlock()
                raise ex

        self.unlock()
        return effect_rows

    def query(self, sql):
        self.lock()

        if (not self.__ispersist):
            try:
                self.connect()
            except Exception as ex:
                self.unlock()
                raise ex
        else:
            try:
                self.__conn.ping(True)
            except Exception as ex:
                self.log_error(str(ex))
                try:
                    self.connect()
                except Exception as ex:
                    self.unlock()
                    raise ex

        result = None
        try:
            self.__cur.execute(sql)
            result = self.__cur.fetchall()
            #self.__conn.commit()
        except Exception as ex:
            self.unlock()
            self.log_error('db query exception: ' + str(ex))
            raise ex

        if (not self.__ispersist):
            try:
                self.close()
            except Exception as ex:
                self.unlock()
                raise ex

        self.unlock()
        return result

    def __enter__(self):
        self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()


'''
rows = Orm(db).table("service").where("allow_stamp_range", 300).where("is_chec_timestamp", 0).where("id", 1).get()

Orm(db).table("service").data([
    {
        "service_name": "orm7",
        'is_check_ip':0,
    },
    {
        "service_name": "orm8",
        'is_check_ip':0,
    },
]).insert()
'''
'''
Orm(db).table("service").data({
    "service_name": "orm10",
    'is_check_ip':0,
}).insert()
'''
'''
Orm(db).table('service').where("id", 37).delete()
'''


class Orm:

    ORDER_ASC = 'asc'
    ORDER_DESC = 'desc'

    def __init__(self, agileutilDb):
        self.db = agileutilDb
        self.tableName = ''
        self.whereCon = ''
        self.updateData = None

    def table(self, tableName):
        self.tableName = tableName
        return self

    def where(self, *args):
        colName = ''
        data = ''
        compareTag = ''

        if len(args) == 0:
            return self
        elif len(args) == 1:
            raise Exception('where param length must be >= 2')
        elif len(args) == 2:
            compareTag = '='
            colName = args[0]
            data = args[1]
        elif len(args) == 3:
            colName = args[0]
            compareTag = args[1]
            data = args[2]
        else:
            raise Exception('where param length not support')

        if self.whereCon != '':
            self.whereCon = self.whereCon + ' and `%s` %s' % (colName,
                                                              compareTag)
        else:
            self.whereCon = ' where `%s` %s' % (colName, compareTag)

        if (type(data) == int) or (type(data) == float):
            self.whereCon = self.whereCon + " %s " % data
        else:
            data = pymysql.escape_string(data)
            self.whereCon = self.whereCon + " '%s' " % data

        return self

    """
    def where(self, colName, data):
        if self.whereCon != '':
            self.whereCon = self.whereCon + ' and `%s` =' % colName
        else:
            self.whereCon = ' where `%s` =' % colName
        if type(data) == int:
            self.whereCon = self.whereCon + " %s "  % data
        else:
            data = pymysql.escape_string(data) 
            self.whereCon = self.whereCon + " '%s' " % data
        return self
    """

    def getSql(self, cmd):
        #cmd 'insert, delete, update, select'
        sql = ''
        if cmd == 'select':
            sql = 'select * from %s %s' % (self.tableName, self.whereCon)
        if cmd == 'insert':
            sql_list = []
            if type(self.updateData) == list:
                sql = 'insert into `%s`' % self.tableName
                for row in self.updateData:
                    cols = []
                    for k, v in row.items():
                        cols.append("`" + k + "`")
                    col = ','.join(cols)
                    sql = sql + '(' + col + ') values'
                    break

                values = []
                for row in self.updateData:
                    tup = []
                    for k, v in row.items():
                        if type(v) == int:
                            tup.append(str(v))
                        elif type(v) == str:
                            tup.append("'" + v + "'")
                    tupStr = '(' + ','.join(tup) + ')'
                    values.append(tupStr)
                valueStr = ', '.join(values)
                sql = sql + valueStr
                self.whereCon = ''
                return sql
            if type(self.updateData) == dict:
                cols = []
                for k, v in self.updateData.items():
                    cols.append("`" + k + "`")
                sql = 'insert into %s' % (self.tableName) + '(' + ','.join(
                    cols) + ') values('
                for k, v in self.updateData.items():
                    if type(v) == int:
                        sql = sql + str(v) + ','
                    if type(v) == str:
                        sql = sql + "'" + v + "',"
                sql = sql[0:-1]
                sql = sql + ')'
                self.whereCon = ''
                return sql
        if cmd == 'delete':
            sql = """delete from %s %s""" % (self.tableName, self.whereCon)
        if cmd == 'update':
            sql = "update %s set" % (self.tableName)
            for k, v in self.updateData.items():
                if type(v) == int:
                    sql = sql + """ %s=%s,""" % ("`" + k + "`", v)
                if type(v) == str:
                    sql = sql + """ %s='%s',""" % ("`" + k + "`", v)
            sql = sql[0:-1]
            sql = sql + ' ' + self.whereCon
        if cmd == 'count':
            sql = """select count(*) as cnt from %s %s""" % (self.tableName,
                                                             self.whereCon)

        self.whereCon = ''
        return sql

    def data(self, updateData):
        if type(updateData) == list:
            for i in range(len(updateData)):
                for k, v in updateData[i].items():
                    if type(v) == str:
                        updateData[i][k] = pymysql.escape_string(v)
        elif type(updateData) == dict:
            for k, v in updateData.items():
                if type(v) == str:
                    updateData[k] = pymysql.escape_string(v)
        else:
            pass
        self.updateData = updateData
        return self

    def get(self):
        sql = self.getSql('select')
        return self.db.query(sql)

    def first(self):
        rows = self.get()
        if len(rows) > 0:
            row = rows[0]
            return row
        return None

    def update(self, retSql=False):
        sql = self.getSql('update')
        return self.db.update(sql)

    def insert(self):
        sql = self.getSql('insert')
        self.db.update(sql)

    def delete(self):
        sql = self.getSql('delete')
        return self.db.update(sql)

    def count(self):
        sql = self.getSql('count')
        rows = self.db.query(sql)
        row = rows[0]
        num = int(row['cnt'])
        return num

    def limit(self, limitNum):
        if not str(limitNum).isdigit():
            raise Exception('param limit is not a number')
        if '.' in str(limitNum):
            raise Exception('param limit is not a integer, but a float')
        limitNum = int(limitNum)
        if limitNum < 0: raise Exception('param limit < 0')
        self.whereCon = self.whereCon + " limit %s" % limitNum
        return self

    def orderBy(self, colName, order=ORDER_ASC):
        if order not in ['asc', 'desc', 'ASC', 'DESC']:
            raise Exception('order by order is invalid')
        self.whereCon = self.whereCon + " order by `%s` %s" % (colName, order)
        return self


class DBCluster(object):
    def __init__(self, masterDb, slaveDbList=[]):
        self._masterDb = masterDb
        self._slaveDbList = slaveDbList
        self._slaveDbCount = len(slaveDbList)
        self._curSlave = 0

    def query(self, sql):
        if len(self._slaveDbList) == 0:
            return self._masterDb.query(sql)
        self._curSlave = self._curSlave + 1
        slaveIndex = self._curSlave % self._slaveDbCount
        slaveDb = self._slaveDbList[slaveIndex]
        return slaveDb.query(sql)

    def queryInstance(self):
        self._curSlave = self._curSlave + 1
        slaveIndex = self._curSlave % self._slaveDbCount
        return slaveIndex

    def update(self, sql):
        return self._masterDb.update(sql)


class DBProxy(object):
    def __init__(self, masterDb, slaveDbList=[]):
        self._masterDb = masterDb
        self._slaveDbList = slaveDbList
        self._slaveDbCount = len(slaveDbList)
        self._curSlave = 0

    def query(self, sql):
        if len(self._slaveDbList) == 0:
            return self._masterDb.query(sql)
        self._curSlave = self._curSlave + 1
        slaveIndex = self._curSlave % self._slaveDbCount
        slaveDb = self._slaveDbList[slaveIndex]
        return slaveDb.query(sql)

    def queryInstance(self):
        self._curSlave = self._curSlave + 1
        slaveIndex = self._curSlave % self._slaveDbCount
        return slaveIndex

    def update(self, sql):
        return self._masterDb.update(sql)


'''
from pymongo import MongoClient

class MongoCollection:

    def __init__(self, coll_name, db):
        self._coll_name = coll_name
        self._db = db
        self._coll = self._db[coll_name]

    def add(self, obj):
        effect_num = 0
        obj_type = type(obj)
        if obj_type != list and obj_type != dict: return 0
        to_add_list = []
        if obj_type == list: to_add_list = obj
        else: to_add_list.append(obj)
        for item in to_add_list:
            self._coll.insert(item)
            effect_num = effect_num + 1
        return effect_num

    def delete(self, condition):
        ret = self._coll.remove(condition)
        effect_num = ret['n']
        return effect_num

    def update(self, condition, to_update, multi = True):
        to_update = {"$set" : to_update}
        ret = self._coll.update(condition, to_update, multi= multi)
        effect_num = ret['nModified']
        return effect_num

    def find_one(self, condition):
        return self._coll.find_one(condition)

    def find(self, condition, limit_cnt = -1, skip_cnt = -1):
        rows = []
        if limit_cnt > 0 and skip_cnt > 0:
            rows = [row for row in self._coll.find(condition).limit(limit_cnt).skip(skip_cnt)]
        elif limit_cnt > 0:
            rows = [row for row in self._coll.find(condition).limit(limit_cnt)]
        elif skip_cnt > 0:
            rows = [row for row in self._coll.find(condition).skip(skip_cnt)]
        else:
            rows = [row for row in self._coll.find(condition)]
        return rows

class MongoDB:

    def __init__(self, db_name, conn):
        self._db_name = db_name
        self._conn = conn
        self._db = self._conn[db_name]

    def select_coll(self, collection_name):
        coll = MongoCollection(collection_name, self._db)
        return coll

    def delete_coll(self, coll_name):
        ret = self._db.drop_collection(coll_name)
        return self

class Mongo:

    def __init__(self, host, port = 27017):
        self._host = host
        self._port = port
        self._conn = MongoClient(self._host, self._port)

    def select_db(self, db_name):
        db = MongoDB(db_name, self._conn)
        return db
'''



#coding=utf-8

import pickle
import json
from abc import ABCMeta, abstractmethod


class RpcSerialize(object):

    __metaclass__ = ABCMeta

    def __init__(self):
        pass
    
    @abstractmethod
    def serialize(cls, obj) -> bytes:
        pass

    @abstractmethod
    def unserialize(cls, bytearr):
        pass


class BinarySerialize(RpcSerialize):

    def __init__(self):
        pass

    @classmethod
    def serialize(cls, obj) -> bytes:
        return pickle.dumps(obj)

    @classmethod
    def unserialize(cls, bytearr):
        return pickle.loads(bytearr)


class JsonSerialize(RpcSerialize):

    def __init__(self):
        pass

    @classmethod
    def serialize(cls, obj) -> bytes:
        return json.dumps(obj)

    @classmethod
    def unserialize(cls, bytearr):
        return json.loads(bytearr)


#coding=utf-8

import multiprocessing
import time


class Worker(object):

    __slots__ = ['_target', '_args', '_proc']

    def __init__(self, target, args):
        self._target = target
        self._args = args
        self._proc = None

    def start(self):
        self._proc = multiprocessing.Process(target=self._target,
                                             args=(self._args, ))
        self._proc.start()

    def stop(self):
        self._proc.terminate()

    def join(self, second):
        self._proc.join(second)

    def terminate(self):
        self._proc.terminate()


def cronMain(params):
    target = params[0]
    args = params[1]
    interval = params[2]
    timeout = params[3]
    while True:
        time.sleep(interval)
        w = Worker(target, (args, ))
        w.start()
        if timeout < 0:
            w.join()
        else:
            w.join(timeout)
        w.terminate()


def execLoop(target, args, interval, timeout=-1):
    if interval <= 0: raise Exception("interval params should be > 0")
    if timeout > 0:
        if interval < timeout: raise Exception("interval < timeout")
    manager = Worker(cronMain, (target, args, interval, timeout))
    manager.start()



#!/usr/bin/env python
'''
pub.sh

#!/bin/sh
rm -rf ./agileutil.egg-info
rm -rf ./build
rm -rf ./dist
rm -rf ./*.tar.gz
python setup.py sdist
twine upload -u [username] -p [password] dist/*
python setup.py install
'''

DEFINE_VERSION = '0.0.7'
from setuptools import setup
import platform
system = platform.system()
webpy = 'web.py'
if platform.python_version()[0:1] == '3':
    webpy = 'web.py==0.40.dev1'
requireList = [
    'requests==2.25.1',
    'PyMySQL==0.10.1',
    'DBUtils==1.3', 
    'sanic==20.12.2',
    'tornado==6.1'
]
#if requireList == 'Linux':
#    requireList.append('japronto')
#    requireList.append('sanic')

setup(name='agileutil',
      version=DEFINE_VERSION,
      description='Light, concurrent RPC framework for Python',
      author='tank',
      license='MIT',
      platforms="any",
      install_requires=requireList,
      classifiers=[
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
      ],
      keywords='agileutil',
      packages=['agileutil', 'agileutil/rpc', 'agileutil/algorithm'],
      include_package_data=True)



#coding=utf-8

import requests
import json
import os

class ZabbixAPI:

    def __init__(self, baseurl, username, password):
        self.baseurl = baseurl
        self.username = username
        self.password = password
        self.timeout = 10
        self.headers = {
            'Content-Type': 'application/json-rpc'
        }
        self.token = None
        self.url = self.baseurl + '/zabbix/api_jsonrpc.php'

    def get_token(self):
        '''
        根据username,password获取token
        :return: str
        '''
        if self.token != None:
            return self.token
        data = {
            "jsonrpc": "2.0",
            "method": "user.login",
            "params": {
                "user": self.username,
                "password": self.password
            },
            "id": 1,
            "auth": None
        }
        r = requests.post(url=self.url, headers=self.headers, data=json.dumps(data), timeout = self.timeout)
        self.token = r.json()['result']
        return self.token

    def __del__(self):
        self.logout()

    def logout(self):
        '''
        注销登录
        :return: bool
        '''
        if self.token == None:
            return
        data = {
            'jsonrpc': '2.0',
            'method': 'user.logout',
            'params': [],
            'id': 1,
            'auth': self.token
        }
        r = requests.post(url=self.url, headers=self.headers, data=json.dumps(data), timeout = self.timeout)
        if r.json()['result'] == True:
            self.token = None
            print('logout true')
            return True
        else:
            raise Exception('logout fail ' + r.text)

    def get_hostid_info_by_field(self, fileld_name, item_list):
        data = {
            'jsonrpc': '2.0',
            'method': 'host.get',
            'params': {
                'output': [
                    'hostid',
                    'host',
                ],
                'filter' : {
                    fileld_name : item_list,
                }
            },
            'auth': self.get_token(),
            'id': 1
        }
        r = requests.post(url=self.url, headers=self.headers, data=json.dumps(data), timeout = self.timeout)
        res = r.json()
        if res.has_key('result'):
            result = res['result']
            if len(result) > 0:
                return result
        raise Exception('host not found' + str(item_list))

    def get_hostid_by_host(self, hostname):
        '''
        根据主机名，获取zabbix中的hostid
        :param hostname: str
        :return: str
        '''
        hostlist = self.get_hostid_info_by_field('host', [ hostname ])
        if len(hostlist)  == 0:
            raise Exception('hostname not found')
        host_dict = hostlist[0]
        if host_dict.get('host', '') == hostname:
            return host_dict.get('hostid')
        raise Exception('hostname not found')

    def get_items_by_host(self, hostname, output = [], fmt = None):
        hostid = self.get_hostid_by_host(hostname)
        data = {
            'jsonrpc': '2.0',
            'method': 'item.get',
            'params': {
                'hostids': hostid,
                'search': {
                    #key: value,
                }
            },
            "id": 1,
            "auth": self.get_token(),
        }
        if len(output) > 0:
            data['params']['output'] = output
        r = requests.post(url=self.url, headers=self.headers, data=json.dumps(data), timeout=self.timeout)
        if fmt == 'json':
            return r.text
        return r.json()['result']

    def get_triggers_by_itemids(self, itemids = [], output = [], fmt=None):
        if len(itemids) == 0:
            return []
        data = {
            'jsonrpc': '2.0',
            'method': 'trigger.get',
            'params': {
                'itemids': itemids,
                'selectFunctions': 'extend'
            },
            "id": 1,
            "auth": self.get_token(),
        }
        if len(output) > 0:
            data['params']['output'] = output
        print('data:', data)
        r = requests.post(url=self.url, headers=self.headers, data=json.dumps(data), timeout=self.timeout)
        if fmt == 'json':
            return r.text
        return r.json()['result']
    
    '''
    def get_host_triggers(self, hostname):
        data = {
            'jsonrpc': '2.0',
            'method': 'trigger.get',
            'params': {
                #'output': [
                #    'triggerid',
                #    'expression',
                #    'description',
                #],
                'filter': {
                    'host': hostname,
                }
            },
            'auth': self.get_token(),
            'id': 1
        }
        r = requests.post(url=self.url, headers=self.headers, data=json.dumps(data), timeout=self.timeout)
        print(r.text, r.status_code)
    '''


    def update_trigger(self, trigger_id, expr):
        data = {
            'jsonrpc': '2.0',
            'method': 'trigger.update',
            'params': {
                "triggerid": trigger_id,
                "expression": expr,
            },
            'auth': self.get_token(),
            'id': 1
        }
        r = requests.post(url=self.url, headers=self.headers, data=json.dumps(data), timeout=self.timeout)
        print(r.text, r.status_code)


if __name__ == '__main__':
    zapi = ZabbixAPI('http://192.168.21.117', 'xxxx', 'xxxx')
    #获取token
    print( zapi.get_token() )
    #根据主机名获取id
    hostid = zapi.get_hostid_by_host('xxxx-01')
    print('hostid:', hostid)
    #根据某字段获取主机
    hostids_info = zapi.get_hostid_info_by_field('host', ['xxxx-01', '192.168.0.1'])
    print('hostids_info:', hostids_info)
    #获取某主机的所有监控项
    items = zapi.get_items_by_host('xxxx-01', output=['itemid', 'name', 'key_', 'status'])
    #获取服务的监控项
    ser_items = [ elem for elem in items if '_auto_' in elem.get('name') ]
    print('ser_items:', ser_items)
    #获取监控对应的trigger
    itemids = [seri.get('itemid') for seri in ser_items]
    itemids = ['33133']
    print('itemids:', itemids)
    #triggers = zapi.get_triggers_by_itemids(itemids=itemids, output=['triggerid', 'expression', 'description', 'status', 'functions', 'priority'], fmt='json')
    triggers = zapi.get_triggers_by_itemids(itemids=itemids, output=[], fmt='json')
    print('triggers:', triggers)
    expr = """{dddddd:errorlog[/data/logs/dddddd/error.log].count(#1, 30, "lt")}=1"""
    zapi.update_trigger('19037', expr)


#coding=utf-8

import socket


class Tcp:
    def __init__(self):
        self.bindAddr = '0.0.0.0'
        self.bindPort = 80
        self.backlog = 10
        self.s = None

    def connect(self, addr, port):
        self.close()
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.s.connect((addr, port))
        return self.s

    def bind(self):
        self.close()
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.s.bind((self.bindAddr, self.bindPort))
        self.s.listen(self.backlog)

    def close(self):
        if self.s != None:
            try:
                self.s.close()
            except:
                pass
            self.s = None

    def recvn(self, n):
        buffer = []
        needRecv = n
        while True:
            bytes = self.s.recv(needRecv)
            if bytes:
                buffer.append(bytes)
            else:
                break
            length = len(bytes)
            needRecv = n - length
            if needRecv == 0: break
        data = ''.join(buffer)
        return data

    def recvall(self):
        buffer = []
        while True:
            bytes = self.s.recv(1024)
            if bytes:
                buffer.append(bytes)
            else:
                break
        data = ''.join(buffer)
        return data

    @staticmethod
    def recv_all(sock):
        buffer = []
        while True:
            bytes = sock.recv(1024)
            if bytes:
                buffer.append(bytes)
            else:
                break
        data = ''.join(buffer)
        return data

    def send(self, data):
        self.s.send(data)
        return len(data)

    def syncIOLoop(self, callback):
        self.bind()
        while True:
            sock, addr = self.s.accept()
            callback(sock, addr)
            try:
                sock.close()
            except:
                pass



#coding=utf-8
'''

基于高性能sanic框架封装

Usage:

from agileutil.sanic import SanicApp, SanicController

class TestController(SanicController):

    async def handle(self):
        self.logInfo("info")
        self.logError("error")
        self.logWarn("warn")
        port = self.getConfig('PORT', '2000')
        headers = self.headers()
        print('headers:', headers)
        method = self.method()
        print('method:', method)
        queryStr = self.queryString()
        print('queryStr:', queryStr)
        path = self.path()
        print('path:', path)
        print('http version:', self.httpVersion())
        print('body:', self.body())
        print('json:', self.requestJson())
        print('cookies:', self.cookies())
        print('remote_addr:', self.remoteAddr())
        print('form:', self.form())
        print('param1:', self.param('param1'))
        print('param2:', self.param('param2'))
        return str(port)

app = SanicApp(debug=True, worker_num=1, log = './access.log')
app.route('/',  TestController)
app.run()
'''

import uuid
import multiprocessing
from sanic import Sanic
from sanic.response import text, raw
from sanic.views import HTTPMethodView
import ujson
from agileutil.log import Log
try:
    from decouple import config
except:
    pass
import logging
from sanic.log import logger

class SanicController(HTTPMethodView):
    def __init__(self):
        self.req = None
        self.logger = None
        self.paramMap = None
        self.isRaw = False

    async def get(self, req):
        return await self.deal(req)

    async def post(self, req):
        return await self.deal(req)

    async def put(self, req):
        return await self.deal(req)

    async def patch(self, req):
        return await self.deal(req)

    async def delete(self, req):
        return await self.deal(req)

    async def deal(self, req):
        self.req = req
        ret = await self.handle()
        if self.isRaw:
            return raw(ret)
        return text(str(ret))  

    async def handle(self):
        return ''

    def resp(self, errno=0, errmsg='', data=''):
        return ujson.dumps({'errno': errno, 'errmsg': errmsg, 'data': data})

    logger = None

    @classmethod
    def setLogger(cls, logger):
        cls.logger = logger

    @classmethod
    def logInfo(cls, string):
        if cls.logger != None: cls.logger.info(string)

    @classmethod
    def logWarn(cls, string):
        if cls.logger != None: cls.logger.warning(string)

    @classmethod
    def logError(cls, string):
        if cls.logger != None: cls.logger.error(string)

    def getConfig(self, key: str, default: str = ''):
        ret = None
        try:
            ret = config(key)
        except:
            ret = default
        return ret

    def headers(self):
        return dict(self.req.headers)

    def method(self):
        return self.req.method

    def queryString(self):
        return self.req.query_string

    def path(self):
        return self.req.path

    def httpVersion(self):
        return self.req.version

    def body(self):
        return self.req.body

    def requestJson(self):
        try:
            return self.req.json
        except:
            return None

    def cookies(self):
        return self.req.cookies

    def remoteAddr(self):
        return self.req.remote_addr

    def form(self):
        return self.req.form

    def param(self, key: str):
        self.initParams()
        if key in self.paramMap:
            return self.paramMap[key]
        return ''

    def initParams(self):
        if self.paramMap == None:
            self.paramMap = {}
            queryStr = self.queryString()
            if queryStr != None and queryStr != '':
                kvList = queryStr.split('&')
                print('kvList:', kvList)
                for item in kvList:
                    k, v = item.split('=')
                    self.paramMap[k] = v
            form = self.form()
            if form != None:
                for k, v in form.items():
                    self.paramMap[k] = v

    def getRequest(self):
        return self.req


class SanicApp(object):
    def __init__(self,
                 host: str = '0.0.0.0',
                 port: int = 9876,
                 debug: bool = True,
                 worker_num: int = multiprocessing.cpu_count(),
                 log: str = '',
                 conf: str = ''):
        self.host = host
        self.port = port
        self.debug = debug
        self.workerNum = worker_num
        self.app = None
        self.log = log
        self.logger = None
        self.initApp()
        self.initLog()

    def disableLog(self):
        logger.setLevel(logging.WARNING)

    def initApp(self):
        if self.app == None:
            self.app = Sanic(name='agileutil' + str(uuid.uuid1()))

    def initLog(self):
        if self.logger == None and self.log != '':
            self.logger = Log(self.log)

    def getLogger(self):
        self.initLog()
        return self.logger

    def run(self):
        self.initApp()
        self.app.run(host=self.host,
                     port=self.port,
                     debug=self.debug,
                     workers=self.workerNum,
                     access_log=False)

    def route(self, path, className):
        className.setLogger(self.getLogger())
        self.app.add_route(className.as_view(), path)
